// ####################################
// #                                  #
// #       Ping of Death - Bot        #
// #                by                #
// #    Markus Klinge aka Count Floyd #
// #                                  #
// ####################################
//
// Started from the HPB-Bot Alpha Source
// by Botman so Credits for a lot of the basic
// HL Server/Client Stuff goes to him
//
// dll.cpp
//
// Links Functions, handles Client Commands, initializes DLL and misc Stuff

#include "bot_globals.h"


// server command handler
void PODBot_ServerCommand (void);
void PbCmdParser (const char *pcmd, const char *arg1, const char *arg2, const char *arg3, const char *arg4, const char *arg5);
void UserNewroundAll (void);
void GetGameDir (void);
void ShowPBKickBotMenu (edict_t *pEntity, int iMenuNum);


// START of Metamod stuff

enginefuncs_t meta_engfuncs;
gamedll_funcs_t *gpGamedllFuncs;
mutil_funcs_t *gpMetaUtilFuncs;
meta_globals_t *gpMetaGlobals;

META_FUNCTIONS gMetaFunctionTable =
{
   NULL, // pfnGetEntityAPI()
   NULL, // pfnGetEntityAPI_Post()
   GetEntityAPI2, // pfnGetEntityAPI2()
   GetEntityAPI2_Post, // pfnGetEntityAPI2_Post()
   NULL, // pfnGetNewDLLFunctions()
   NULL, // pfnGetNewDLLFunctions_Post()
   GetEngineFunctions, // pfnGetEngineFunctions()
   NULL, // pfnGetEngineFunctions_Post()
};

plugin_info_t Plugin_info = {
   META_INTERFACE_VERSION, // interface version
   "POD-Bot mm", // plugin name
   "R2b46b", // plugin version
   __DATE__, // date of creation
   "Count Floyd & Bots United", // plugin author
   "http://www.bots-united.com", // plugin URL
   "PODBOTMM", // plugin logtag
   PT_CHANGELEVEL, // when loadable
   PT_ANYTIME, // when unloadable
};


C_DLLEXPORT int Meta_Query (char *ifvers, plugin_info_t **pPlugInfo, mutil_funcs_t *pMetaUtilFuncs)
{
   // this function is the first function ever called by metamod in the plugin DLL. Its purpose
   // is for metamod to retrieve basic information about the plugin, such as its meta-interface
   // version, for ensuring compatibility with the current version of the running metamod.

   // keep track of the pointers to metamod function tables metamod gives us
   gpMetaUtilFuncs = pMetaUtilFuncs;
   *pPlugInfo = &Plugin_info;

   // check for interface version compatibility
   if (strcmp (ifvers, Plugin_info.ifvers) != 0)
   {
      int mmajor = 0, mminor = 0, pmajor = 0, pminor = 0;

      LOG_CONSOLE (PLID, "%s: meta-interface version mismatch (metamod: %s, %s: %s)", Plugin_info.name, ifvers, Plugin_info.name, Plugin_info.ifvers);
      LOG_MESSAGE (PLID, "%s: meta-interface version mismatch (metamod: %s, %s: %s)", Plugin_info.name, ifvers, Plugin_info.name, Plugin_info.ifvers);

      // if plugin has later interface version, it's incompatible (update metamod)
      sscanf (ifvers, "%d:%d", &mmajor, &mminor);
      sscanf (META_INTERFACE_VERSION, "%d:%d", &pmajor, &pminor);

      if ((pmajor > mmajor) || ((pmajor == mmajor) && (pminor > mminor)))
      {
         LOG_CONSOLE (PLID, "metamod version is too old for this plugin; update metamod");
         LOG_ERROR (PLID, "metamod version is too old for this plugin; update metamod");
         return (FALSE);
      }

      // if plugin has older major interface version, it's incompatible (update plugin)
      else if (pmajor < mmajor)
      {
         LOG_CONSOLE (PLID, "metamod version is incompatible with this plugin; please find a newer version of this plugin");
         LOG_ERROR (PLID, "metamod version is incompatible with this plugin; please find a newer version of this plugin");
         return (FALSE);
      }
   }

   return (TRUE); // tell metamod this plugin looks safe
}


C_DLLEXPORT int Meta_Attach (PLUG_LOADTIME now, META_FUNCTIONS *pFunctionTable, meta_globals_t *pMGlobals, gamedll_funcs_t *pGamedllFuncs)
{
   // this function is called when metamod attempts to load the plugin. Since it's the place
   // where we can tell if the plugin will be allowed to run or not, we wait until here to make
   // our initialization stuff, like registering CVARs and dedicated server commands.

   // are we allowed to load this plugin now ?
   if (now > Plugin_info.loadable)
   {
      LOG_CONSOLE (PLID, "%s: plugin NOT attaching (can't load plugin right now)", Plugin_info.name);
      LOG_ERROR (PLID, "%s: plugin NOT attaching (can't load plugin right now)", Plugin_info.name);
      return (FALSE); // returning FALSE prevents metamod from attaching this plugin
   }

   // keep track of the pointers to engine function tables metamod gives us
   gpMetaGlobals = pMGlobals;
   memcpy (pFunctionTable, &gMetaFunctionTable, sizeof (META_FUNCTIONS));
   gpGamedllFuncs = pGamedllFuncs;

   // print a message to notify about plugin attaching
   LOG_CONSOLE (PLID, "%s: plugin attaching", Plugin_info.name);
   LOG_MESSAGE (PLID, "%s: plugin attaching", Plugin_info.name);

   // ask the engine to register the server commands this plugin uses
   REG_SVR_COMMAND ((char *)g_pszPbCmds[PBCMD], PODBot_ServerCommand);
   
   return (TRUE); // returning TRUE enables metamod to attach this plugin
}


C_DLLEXPORT int Meta_Detach (PLUG_LOADTIME now, PL_UNLOAD_REASON reason)
{
   // this function is called when metamod unloads the plugin. A basic check is made in order
   // to prevent unloading the plugin if its processing should not be interrupted.

   // is metamod allowed to unload the plugin ?
   if ((now > Plugin_info.unloadable) && (reason != PNL_CMD_FORCED))
   {
      LOG_CONSOLE (PLID, "%s: plugin NOT detaching (can't unload plugin right now)", Plugin_info.name);
      LOG_ERROR (PLID, "%s: plugin NOT detaching (can't unload plugin right now)", Plugin_info.name);
      return (FALSE); // returning FALSE prevents metamod from unloading this plugin
   }

   // Delete all allocated Memory
   BotFreeAllMemory();

   return (TRUE); // returning TRUE enables metamod to unload this plugin
}

// END of Metamod stuff


#ifndef __linux__
BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
   // Required DLL entry point

   return (TRUE);
}
#endif


void WINAPI GiveFnptrsToDll (enginefuncs_t* pengfuncsFromEngine, globalvars_t *pGlobals)
{
   // get the engine functions from the engine...

   memcpy (&g_engfuncs, pengfuncsFromEngine, sizeof (enginefuncs_t));
   gpGlobals = pGlobals;
}


void GameDLLInit (void)
{
   // First function called from HL in our replacement Server DLL

   unsigned char *tempbuf;
   int tempsize;
   g_bIsDedicatedServer = (IS_DEDICATED_SERVER () > 0);

   if (!g_bIsDedicatedServer)
      g_iPeoBotsKept = 1;

   GetGameDir ();

   // Counter-Strike 1.5 detection
   tempbuf = LOAD_FILE_FOR_ME ("models/w_famas.mdl", &tempsize);
   if (tempbuf != NULL)
   {
      g_bIsOldCS15 = FALSE;
      FREE_FILE (tempbuf);
   }
   else
      g_bIsOldCS15 = TRUE;

   // Reset the bot creation tab
   memset (BotCreateTab, 0, sizeof (BotCreateTab));

   // open the bot config file
   file_opened = FALSE;
   g_GameRules = TRUE;

   RETURN_META (MRES_IGNORED);
}


int Spawn (edict_t *pent)
{
   // Something gets spawned in the game

   if (strcmp (STRING (pent->v.classname), "worldspawn") == 0)
   {
      g_iMapType = 0; // reset g_iMapType as worldspawn is the first entity spawned
      PRECACHE_SOUND ("weapons/xbow_hit1.wav"); // waypoint add
      PRECACHE_SOUND ("weapons/mine_activate.wav"); // waypoint delete
      PRECACHE_SOUND ("common/wpn_hudon.wav"); // path add/delete done
      PRECACHE_SOUND ("debris/bustglass1.wav"); // waypoint error found
      m_spriteTexture = PRECACHE_MODEL ("sprites/lgtning.spr");
   }

   else if (strcmp (STRING (pent->v.classname), "info_player_start") == 0)
   {
      SET_MODEL (pent, "models/player/urban/urban.mdl");
      pent->v.rendermode = kRenderTransAlpha; // set its render mode to transparency
      pent->v.renderamt = 127; // set its transparency amount
      pent->v.effects |= EF_NODRAW;
   }

   else if (strcmp (STRING (pent->v.classname), "info_player_deathmatch") == 0)
   {
      SET_MODEL (pent, "models/player/terror/terror.mdl");
      pent->v.rendermode = kRenderTransAlpha; // set its render mode to transparency
      pent->v.renderamt = 127; // set its transparency amount
      pent->v.effects |= EF_NODRAW;
   }

   else if (strcmp (STRING (pent->v.classname), "info_vip_start") == 0)
   {
      SET_MODEL (pent, "models/player/vip/vip.mdl");
      pent->v.rendermode = kRenderTransAlpha; // set its render mode to transparency
      pent->v.renderamt = 127; // set its transparency amount
      pent->v.effects |= EF_NODRAW;
   }

   else if ((strcmp (STRING (pent->v.classname), "func_vip_safetyzone") == 0)
            || (strcmp (STRING (pent->v.classname), "info_vip_safetyzone") == 0))
      g_iMapType |= MAP_AS; // assassination map

   else if (strcmp (STRING (pent->v.classname), "hostage_entity") == 0)
      g_iMapType |= MAP_CS; // rescue map

   else if ((strcmp (STRING (pent->v.classname), "func_bomb_target") == 0)
            || (strcmp (STRING (pent->v.classname), "info_bomb_target") == 0))
      g_iMapType |= MAP_DE; // defusion map

   RETURN_META_VALUE (MRES_IGNORED, 0);
}


int Spawn_Post (edict_t *pent)
{
   // solves the bots unable to see through certain types of glass bug.
   // MAPPERS: NEVER EVER ALLOW A TRANSPARENT ENTITY TO WEAR THE FL_WORLDBRUSH FLAG !!!

   if (pent->v.rendermode == kRenderTransTexture)
      pent->v.flags &= ~FL_WORLDBRUSH; // clear the FL_WORLDBRUSH flag out of transparent ents

   RETURN_META_VALUE (MRES_IGNORED, 0);
}


BOOL ClientConnect (edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ])
{
   // Client connects to this Server

   // check if this client is the listen server client
   if (FStrEq (pszAddress, "loopback"))
      pHostEdict = pEntity; // save the edict of the listen server client...

   RETURN_META_VALUE (MRES_IGNORED, 0);
}


void ClientDisconnect (edict_t *pEntity)
{
   // Client disconnects from this Server

   int i;

   i = ENTINDEX (pEntity) - 1;

   // Find & remove this Client from our list of Clients connected
   clients[i].welcome_time = 0;
   clients[i].wptmessage_time = 0;

   // Check if its a Bot
   if (bots[i].pEdict == pEntity)
   {
      // Delete Nodes from Pathfinding
      DeleteSearchNodes (&bots[i]);
      BotResetTasks (&bots[i]);
      memset (&bots[i], 0, sizeof (bot_t));
      memset (&clients[i], 0, sizeof (client_t));
      bots[i].is_used = FALSE; // this slot is now free to use
      bots[i].f_kick_time = gpGlobals->time; // save the kicked time
   }

   // Check if its the Host disconnecting
   if (pEntity == pHostEdict)
      pHostEdict = NULL;

   RETURN_META (MRES_IGNORED);
}


void ClientPutInServer (edict_t *pEntity)
{
   // Client is finally put into the Server

   int i;

   i = ENTINDEX (pEntity) - 1;

   if (g_bShowWelcomeMessages)
   {
      clients[i].welcome_time = gpGlobals->time + 10.0;
      clients[i].wptmessage_time = gpGlobals->time + 1.0;
   }

   RETURN_META (MRES_IGNORED);
}


void ClientCommand (edict_t *pEntity)
{
   // Executed if a client typed some sort of command into the console

   static char pcmd[128];
   static char arg1[128];
   static char arg2[128];
   static char arg3[128];
   static char arg4[128];
   static char arg5[128];
   static char arg6[128];
   static char arg7[128];
   static char arg8[128];
   static char arg9[128];
   edict_t *pSpawnPoint = NULL;
   int iClientIndex;
   int iRadioCommand;
   int i;
   char szTemp[1024];

   snprintf (pcmd, sizeof (pcmd), CMD_ARGV (0));
   snprintf (arg1, sizeof (arg1), CMD_ARGV (1));
   snprintf (arg2, sizeof (arg2), CMD_ARGV (2));
   snprintf (arg3, sizeof (arg3), CMD_ARGV (3));
   snprintf (arg4, sizeof (arg4), CMD_ARGV (4));
   snprintf (arg5, sizeof (arg5), CMD_ARGV (5));
   snprintf (arg6, sizeof (arg6), CMD_ARGV (6));
   snprintf (arg7, sizeof (arg7), CMD_ARGV (7));
   snprintf (arg8, sizeof (arg8), CMD_ARGV (8));
   snprintf (arg9, sizeof (arg9), CMD_ARGV (9));

   // don't search ClientCommands of Bots or other Edicts than Host!
   if (!isFakeClientCommand && (pEntity == pHostEdict) && !g_bIsDedicatedServer)
   {
      // "pb" Check if any were the Podbot commands..
      if (FStrEq (pcmd, g_pszPbCmds[PBCMD]))
      {
         PbCmdParser(arg1, arg2, arg3, arg4, arg5, arg6);
         RETURN_META (MRES_SUPERCEDE);
      }

      // "waypoint"
      else if (FStrEq (pcmd, CONSOLE_CMD_WAYPOINT)
               || FStrEq (pcmd, CONSOLE_CMD_WP))
      {
         if (FStrEq (arg1, "stats"))
         {
            int iButtonPoints = 0;
            int iLiftPoints = 0;
            int iCrouchPoints = 0;
            int iCrossingPoints = 0;
            int iGoalPoints = 0;
            int iLadderPoints = 0;
            int iRescuePoints = 0;
            int iCampPoints = 0;
            int iNoHostagePoints = 0;
            int iTerroristPoints = 0;
            int iCounterPoints = 0;
            int iNormalPoints = 0;

            for (i = 0; i < g_iNumWaypoints; i++)
            {
               if (paths[i]->flags & W_FL_USE_BUTTON)
                  iButtonPoints++;
               if (paths[i]->flags & W_FL_LIFT)
                  iLiftPoints++;
               if (paths[i]->flags & W_FL_CROUCH)
                  iCrouchPoints++;
               if (paths[i]->flags & W_FL_CROSSING)
                  iCrossingPoints++;
               if (paths[i]->flags & W_FL_GOAL)
                  iGoalPoints++;
               if (paths[i]->flags & W_FL_LADDER)
                  iLadderPoints++;
               if (paths[i]->flags & W_FL_RESCUE)
                  iRescuePoints++;
               if (paths[i]->flags & W_FL_CAMP)
                  iCampPoints++;
               if (paths[i]->flags & W_FL_NOHOSTAGE)
                  iNoHostagePoints++;
               if (paths[i]->flags & W_FL_TERRORIST)
                  iTerroristPoints++;
               if (paths[i]->flags & W_FL_COUNTER)
                  iCounterPoints++;
               if (paths[i]->flags == 0)
                  iNormalPoints++;
            }

            UTIL_ServerPrint ("Waypoint Statistics:\n");
            UTIL_ServerPrint ("--------------------\n");
            UTIL_ServerPrint ("Waypoints classification per flag:\n");
            UTIL_ServerPrint ("W_FL_USE_BUTTON: %d\n", iButtonPoints);
            UTIL_ServerPrint ("W_FL_LIFT: %d\n", iLiftPoints);
            UTIL_ServerPrint ("W_FL_CROUCH: %d\n", iCrouchPoints);
            UTIL_ServerPrint ("W_FL_CROSSING: %d\n", iCrossingPoints);
            UTIL_ServerPrint ("W_FL_GOAL: %d\n", iGoalPoints);
            UTIL_ServerPrint ("W_FL_LADDER: %d\n", iLadderPoints);
            UTIL_ServerPrint ("W_FL_RESCUE: %d\n", iRescuePoints);
            UTIL_ServerPrint ("W_FL_CAMP: %d\n", iCampPoints);
            UTIL_ServerPrint ("W_FL_NOHOSTAGE: %d\n", iNoHostagePoints);
            UTIL_ServerPrint ("W_FL_TERRORIST: %d\n", iTerroristPoints);
            UTIL_ServerPrint ("W_FL_COUNTER: %d\n", iCounterPoints);
            UTIL_ServerPrint ("Not flagged: %d\n", iNormalPoints);
            UTIL_ServerPrint ("--------------------\n");
            UTIL_ServerPrint ("Total number of waypoints: %d\n", g_iNumWaypoints);
         }
         else if (FStrEq (arg1, "teleport"))
         {
            if ((atoi (arg2) >= 0) && (atoi (arg2) < g_iNumWaypoints))
            {
               TraceResult tr;
               TRACE_HULL (paths[atoi (arg2)]->origin + Vector (0, 0, 32),
                           paths[atoi (arg2)]->origin + Vector (0, 0, -32),
                           ignore_monsters, human_hull, pEntity, &tr);
               SET_ORIGIN (pEntity, tr.vecEndPos);
            }
         }
         else
         {
            UTIL_ServerPrint ("Unknown waypoint command.\n");
            UTIL_ServerPrint ("waypoint commands are: stats teleport\n");
         }

         RETURN_META (MRES_SUPERCEDE);
      }

      // "podbotmenu"
      else if (FStrEq (pcmd, CONSOLE_CMD_PODMENU))
      {
         UTIL_ShowMenu (pEntity, &menuPODBotMain);
         RETURN_META (MRES_SUPERCEDE);
      }

      // "wpmenu"
      else if (FStrEq (pcmd, CONSOLE_CMD_WPMENU))
      {
         UTIL_DisplayWpMenuWelcomeMessage ();
         UTIL_ShowMenu (pEntity, &menuWpMain);
         RETURN_META (MRES_SUPERCEDE);
      }

      // Care for Menus instead...
      else if (pUserMenu != NULL)
      {
         if (FStrEq (pcmd, "menuselect"))
         {
            // Waypoint Main Menu
            if (pUserMenu == &menuWpMain)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
               {
                  g_bWaypointOn = TRUE; // turn waypoints on if off
                  UTIL_ShowMenu (pEntity, &menuWpAdd);
               }
               else if (FStrEq (arg1, "2"))
               {
                  g_bWaypointOn = TRUE; // turn waypoints on if off
                  UTIL_ShowMenu (pEntity, &menuWpDelete);
               }
               else if (FStrEq (arg1, "3"))
               {
                  g_bWaypointOn = TRUE; // turn waypoints on if off
                  UTIL_ShowMenu (pEntity, &menuWpSetRadius);
               }
               else if (FStrEq (arg1, "4"))
               {
                  g_bWaypointOn = TRUE; // turn waypoints on if off
                  UTIL_ShowMenu (pEntity, &menuWpSetFlags);
               }
               else if (FStrEq (arg1, "5"))
               {
                  g_bWaypointOn = TRUE; // turn waypoints on if off
                  UTIL_ShowMenu (pEntity, &menuWpAddPath);
               }
               else if (FStrEq (arg1, "6"))
               {
                  g_bWaypointOn = TRUE; // turn waypoints on if off
                  UTIL_ShowMenu (pEntity, &menuWpDeletePath);
               }
               else if (FStrEq (arg1, "7"))
               {
                  if (WaypointNodesValid ())
                     UTIL_HostPrint ("All Nodes work fine !\n");
               }
               else if (FStrEq (arg1, "8"))
               {
                  if (WaypointNodesValid ())
                  {
                     WaypointSave ();
                     UTIL_HostPrint ("Waypoints saved!\n");
                  }
                  else
                     UTIL_ShowMenu (pEntity, &menuWpSave);
               }
               else if (FStrEq (arg1, "9"))
                  UTIL_ShowMenu (pEntity, &menuWpOptions);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint Add Menu
            else if (pUserMenu == &menuWpAdd)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
                  WaypointAdd (WAYPOINT_ADD_NORMAL); // normal
               else if (FStrEq (arg1, "2"))
                  WaypointAdd (WAYPOINT_ADD_TERRORIST); // t important
               else if (FStrEq (arg1, "3"))
                  WaypointAdd (WAYPOINT_ADD_COUNTER); // ct important
               else if (FStrEq (arg1, "4"))
                  WaypointAdd (WAYPOINT_ADD_LADDER); // ladder
               else if (FStrEq (arg1, "5"))
                  WaypointAdd (WAYPOINT_ADD_RESCUE); // rescue
               else if (FStrEq (arg1, "6"))
                  WaypointAdd (WAYPOINT_ADD_CAMP_START); // camp start
               else if (FStrEq (arg1, "7"))
                  WaypointAdd (WAYPOINT_ADD_CAMP_END); // camp end
               else if (FStrEq (arg1, "8"))
                  WaypointAdd (WAYPOINT_ADD_GOAL); // goal
               else if (FStrEq (arg1, "9"))
               {
                  g_bLearnJumpWaypoint = TRUE;
                  UTIL_HostPrint ("Observation on !\n");
                  if (g_bUseSpeech)
                     SERVER_COMMAND ("speak \"movement check ok\"\n");
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint Delete Menu
            else if (pUserMenu == &menuWpDelete)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
                  WaypointDelete ();

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint SetRadius Menu
            else if (pUserMenu == &menuWpSetRadius)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
                  WaypointChangeRadius (0);
               else if (FStrEq (arg1, "2"))
                  WaypointChangeRadius (8);
               else if (FStrEq (arg1, "3"))
                  WaypointChangeRadius (16);
               else if (FStrEq (arg1, "4"))
                  WaypointChangeRadius (32);
               else if (FStrEq (arg1, "5"))
                  WaypointChangeRadius (48);
               else if (FStrEq (arg1, "6"))
                  WaypointChangeRadius (64);
               else if (FStrEq (arg1, "7"))
                  WaypointChangeRadius (80);
               else if (FStrEq (arg1, "8"))
                  WaypointChangeRadius (96);
               else if (FStrEq (arg1, "9"))
                  WaypointChangeRadius (112);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint SetFlags Menu
            else if (pUserMenu == &menuWpSetFlags)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
                  WaypointChangeFlag (W_FL_USE_BUTTON, FLAG_TOGGLE);
               else if (FStrEq (arg1, "2"))
                  WaypointChangeFlag (W_FL_LIFT, FLAG_TOGGLE);
               else if (FStrEq (arg1, "3"))
                  WaypointChangeFlag (W_FL_CROUCH, FLAG_TOGGLE);
               else if (FStrEq (arg1, "4"))
                  WaypointChangeFlag (W_FL_GOAL, FLAG_TOGGLE);
               else if (FStrEq (arg1, "5"))
                  WaypointChangeFlag (W_FL_LADDER, FLAG_TOGGLE);
               else if (FStrEq (arg1, "6"))
                  WaypointChangeFlag (W_FL_RESCUE, FLAG_TOGGLE);
               else if (FStrEq (arg1, "7"))
                  WaypointChangeFlag (W_FL_CAMP, FLAG_TOGGLE);
               else if (FStrEq (arg1, "8"))
                  WaypointChangeFlag (W_FL_NOHOSTAGE, FLAG_TOGGLE);
               else if (FStrEq (arg1, "9"))
                  UTIL_ShowMenu (pEntity, &menuWpSetTeam);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint Set Team Menu
            else if (pUserMenu == &menuWpSetTeam)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
               {
                  WaypointChangeFlag (W_FL_TERRORIST, FLAG_SET);
                  WaypointChangeFlag (W_FL_COUNTER, FLAG_CLEAR);
               }
               else if (FStrEq (arg1, "2"))
               {
                  WaypointChangeFlag (W_FL_TERRORIST, FLAG_CLEAR);
                  WaypointChangeFlag (W_FL_COUNTER, FLAG_SET);
               }
               else if (FStrEq (arg1, "3"))
               {
                  WaypointChangeFlag (W_FL_TERRORIST, FLAG_CLEAR);
                  WaypointChangeFlag (W_FL_COUNTER, FLAG_CLEAR);
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint Add Path Menu
            else if (pUserMenu == &menuWpAddPath)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
                  WaypointCreatePath (PATH_OUTGOING);
               else if (FStrEq (arg1, "2"))
                  WaypointCreatePath (PATH_INCOMING);
               else if (FStrEq (arg1, "3"))
                  WaypointCreatePath (PATH_BOTHWAYS);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint Delete Path Menu
            else if (pUserMenu == &menuWpDeletePath)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
                  WaypointDeletePath ();

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint Save Menu
            else if (pUserMenu == &menuWpSave)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
               {
                  WaypointSave ();
                  UTIL_HostPrint ("WARNING: Waypoints saved with errors!\n");
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint Options Menu
            else if (pUserMenu == &menuWpOptions)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1")) // wp on/off
               {
                  g_bWaypointOn ^= TRUE; // switch variable on/off (XOR it)

                  if (g_bWaypointOn)
                  {
                     UTIL_HostPrint ("Waypoints Editing is ON\n");

                     while (!FNullEnt (pSpawnPoint = FIND_ENTITY_BY_STRING (pSpawnPoint, "classname", "info_player_start")))
                        pSpawnPoint->v.effects &= ~EF_NODRAW;
                     while (!FNullEnt (pSpawnPoint = FIND_ENTITY_BY_STRING (pSpawnPoint, "classname", "info_player_deathmatch")))
                        pSpawnPoint->v.effects &= ~EF_NODRAW;
                     while (!FNullEnt (pSpawnPoint = FIND_ENTITY_BY_STRING (pSpawnPoint, "classname", "info_vip_start")))
                        pSpawnPoint->v.effects &= ~EF_NODRAW;
                  }
                  else
                  {
                     UTIL_HostPrint ("Waypoint Editing turned OFF\n");

                     while (!FNullEnt (pSpawnPoint = FIND_ENTITY_BY_STRING (pSpawnPoint, "classname", "info_player_start")))
                        pSpawnPoint->v.effects |= EF_NODRAW;
                     while (!FNullEnt (pSpawnPoint = FIND_ENTITY_BY_STRING (pSpawnPoint, "classname", "info_player_deathmatch")))
                        pSpawnPoint->v.effects |= EF_NODRAW;
                     while (!FNullEnt (pSpawnPoint = FIND_ENTITY_BY_STRING (pSpawnPoint, "classname", "info_vip_start")))
                        pSpawnPoint->v.effects |= EF_NODRAW;

                     if (g_bWaypointsChanged && g_bWaypointsSaved)
                     {
                        UTIL_HostPrint ("The map will restart in 5 seconds!\n");
                        g_fTimeRestartServer = gpGlobals->time + 4.0;
                     }
                     else if (g_bWaypointsChanged)
                        UTIL_HostPrint ("Don't forget to SAVE your waypoints...\n");
                  }
               }
               else if (FStrEq (arg1, "2"))
               {
                  g_bAutoWaypoint ^= TRUE; // Switch Variable on/off (XOR it)
                  UTIL_HostPrint ("Auto-Waypointing is %s\n", (g_bAutoWaypoint ? "ENABLED" : "DISABLED"));
               }
               else if (FStrEq (arg1, "3"))
               {
                  g_bEditNoclip ^= TRUE; // Switch Variable on/off (XOR it)
                  if (g_bEditNoclip)
                     pHostEdict->v.movetype = MOVETYPE_NOCLIP;
                  else
                     pHostEdict->v.movetype = MOVETYPE_WALK;
                  UTIL_HostPrint ("No Clipping Cheat is %s\n", (g_bEditNoclip ? "ENABLED" : "DISABLED"));
               }
               else if (FStrEq (arg1, "4"))
               {
                  g_bIgnoreEnemies ^= TRUE; // Switch Variable on/off (XOR it)
                  UTIL_HostPrint ("Peace Mode is %s (Bots %signore Enemies)\n", (g_bIgnoreEnemies ? "ENABLED" : "DISABLED"), (g_bIgnoreEnemies ? "" : "DON'T "));
               }
               else if (FStrEq (arg1, "5"))
               {
                  g_bShowWpFlags ^= TRUE; // Switch Variable on/off (XOR it)
                  UTIL_HostPrint ("Waypoint Flag display is %s\n", (g_bShowWpFlags ? "ENABLED" : "DISABLED"));
               }
               else if (FStrEq (arg1, "6"))
                  UTIL_ShowMenu (pEntity, &menuWpAutoPathMaxDistance);
               else if (FStrEq (arg1, "7"))
                  WaypointCache ();

               RETURN_META (MRES_SUPERCEDE);
            }

            // Waypoint AutoPathMaxDistance Menu 
            else if (pUserMenu == &menuWpAutoPathMaxDistance)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

//               UTIL_DisplayWpMenuWelcomeMessage ();

               if (FStrEq (arg1, "1"))
                  g_fAutoPathMaxDistance = 0;
               else if (FStrEq (arg1, "2"))
                  g_fAutoPathMaxDistance = 100;
               else if (FStrEq (arg1, "3"))
                  g_fAutoPathMaxDistance = 130;
               else if (FStrEq (arg1, "4"))
                  g_fAutoPathMaxDistance = 160;
               else if (FStrEq (arg1, "5"))
                  g_fAutoPathMaxDistance = 190;
               else if (FStrEq (arg1, "6"))
                  g_fAutoPathMaxDistance = 220;
               else if (FStrEq (arg1, "7"))
                  g_fAutoPathMaxDistance = 250;

               if (g_fAutoPathMaxDistance == 0)
                  UTIL_HostPrint ("Auto-path disabled\n");
               else
                  UTIL_HostPrint ("Auto-path Max Distance set to %f\n", g_fAutoPathMaxDistance);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Main PODBot Menu ?
            else if (pUserMenu == &menuPODBotMain)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (FStrEq (arg1, "1"))
               {
                  PbCmdParser (g_pszPbCmds[PBCMD_ADDBOT], NULL, NULL, NULL, NULL, NULL);
               }
               else if (FStrEq (arg1, "2"))
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotSkill);
               else if (FStrEq (arg1, "3"))
               {
                  PbCmdParser (g_pszPbCmds[PBCMD_KILLBOTS], NULL, NULL, NULL, NULL, NULL);
               }
               else if (FStrEq (arg1, "4"))
                  UserNewroundAll ();
               else if (FStrEq (arg1, "5"))
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerSkill);
               else if (FStrEq (arg1, "6"))
                  ShowPBKickBotMenu (pEntity, 1);
               else if (FStrEq (arg1, "7"))
               {
                  PbCmdParser (g_pszPbCmds[PBCMD_REMOVEBOTS], NULL, NULL, NULL, NULL, NULL);
               }
               else if (FStrEq (arg1, "8"))
                  UTIL_ShowMenu (pEntity, &menuPODBotWeaponMode);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Add Bot Skill menu ?
            else if (pUserMenu == &menuPODBotAddBotSkill)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (FStrEq (arg1, "1"))
               {
                  snprintf (g_cStoreAddbotSkill, sizeof (g_cStoreAddbotSkill), "%d", RANDOM_LONG (1, 19));
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotPersonality);
               }
               else if (FStrEq (arg1, "2"))
               {
                  snprintf (g_cStoreAddbotSkill, sizeof (g_cStoreAddbotSkill), "%d", RANDOM_LONG (20, 39));
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotPersonality);
               }
               else if (FStrEq (arg1, "3"))
               {
                  snprintf (g_cStoreAddbotSkill, sizeof (g_cStoreAddbotSkill), "%d", RANDOM_LONG (40, 59));
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotPersonality);
               }
               else if (FStrEq (arg1, "4"))
               {
                  snprintf (g_cStoreAddbotSkill, sizeof (g_cStoreAddbotSkill), "%d", RANDOM_LONG (60, 79));
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotPersonality);
               }
               else if (FStrEq (arg1, "5"))
               {
                  snprintf (g_cStoreAddbotSkill, sizeof (g_cStoreAddbotSkill), "%d", RANDOM_LONG (80, 99));
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotPersonality);
               }
               else if (FStrEq (arg1, "6"))
               {
                  g_cStoreAddbotSkill[0] = '1';
                  g_cStoreAddbotSkill[1] = '0';
                  g_cStoreAddbotSkill[2] = '0';
                  g_cStoreAddbotSkill[3] = 0;
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotPersonality);
               }
               else if (FStrEq (arg1, "7"))
               {
                  g_cStoreAddbotSkill[0] = '1';
                  g_cStoreAddbotSkill[1] = '0';
                  g_cStoreAddbotSkill[2] = '1';
                  g_cStoreAddbotSkill[3] = 0;
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotPersonality);
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Add Bot Personality menu ?
            else if (pUserMenu == &menuPODBotAddBotPersonality)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if ((atoi (arg1) >= 1) && (atoi (arg1) <= 3))
               {
                  g_cStoreAddbotPersonality = arg1[0];
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotTeam);
               }
               else if (FStrEq (arg1, "5"))
               {
                  g_cStoreAddbotPersonality = arg1[0];
                  UTIL_ShowMenu (pEntity, &menuPODBotAddBotTeam);
               }


               RETURN_META (MRES_SUPERCEDE);
            }

            // Add Bot Team Select Menu ?
            else if (pUserMenu == &menuPODBotAddBotTeam)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (atoi (arg1) >= 1 && (atoi (arg1) <= 2))
               {
                  g_cStoreAddbotTeam = arg1[0];
                  if (atoi (arg1) == TEAM_CS_TERRORIST)
                     UTIL_ShowMenu (pEntity, &menuPODBotAddBotTModel);
                  else // TEAM_CS_COUNTER
                     UTIL_ShowMenu (pEntity, &menuPODBotAddBotCTModel);
               }
               else if (FStrEq (arg1, "5"))
               {
                  PbCmdParser (g_pszPbCmds[PBCMD_ADDBOT], g_cStoreAddbotSkill, &g_cStoreAddbotPersonality, arg1, NULL, NULL);
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Add Bot Model Selection Menu ?
            else if ((pUserMenu == &menuPODBotAddBotTModel)
                     || (pUserMenu == &menuPODBotAddBotCTModel))
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if ((atoi (arg1) >= 1) && (atoi (arg1) <= 5))
               {
                  PbCmdParser (g_pszPbCmds[PBCMD_ADDBOT], g_cStoreAddbotSkill, &g_cStoreAddbotPersonality, &g_cStoreAddbotTeam, arg1, NULL);
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Fill Server Bot Skill menu ?
            else if (pUserMenu == &menuPODBotFillServerSkill)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (FStrEq (arg1, "1"))
               {
                  snprintf (g_cStoreFillServerSkill, sizeof (g_cStoreFillServerSkill), "%d", RANDOM_LONG (1, 19));
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerPersonality);
               }
               else if (FStrEq (arg1, "2"))
               {
                  snprintf (g_cStoreFillServerSkill, sizeof (g_cStoreFillServerSkill), "%d", RANDOM_LONG (20, 39));
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerPersonality);
               }
               else if (FStrEq (arg1, "3"))
               {
                  snprintf (g_cStoreFillServerSkill, sizeof (g_cStoreFillServerSkill), "%d", RANDOM_LONG (40, 59));
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerPersonality);
               }
               else if (FStrEq (arg1, "4"))
               {
                  snprintf (g_cStoreFillServerSkill, sizeof (g_cStoreFillServerSkill), "%d", RANDOM_LONG (60, 79));
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerPersonality);
               }
               else if (FStrEq (arg1, "5"))
               {
                  snprintf (g_cStoreFillServerSkill, sizeof (g_cStoreFillServerSkill), "%d", RANDOM_LONG (80, 99));
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerPersonality);
               }
               else if (FStrEq (arg1, "6"))
               {
                  g_cStoreFillServerSkill[0] = '1';
                  g_cStoreFillServerSkill[1] = '0';
                  g_cStoreFillServerSkill[2] = '0';
                  g_cStoreFillServerSkill[3] = 0;
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerPersonality);
               }
               else if (FStrEq (arg1, "7"))
               {
                  g_cStoreFillServerSkill[0] = '1';
                  g_cStoreFillServerSkill[1] = '0';
                  g_cStoreFillServerSkill[2] = '1';
                  g_cStoreFillServerSkill[3] = 0;
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerPersonality);
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Fill Server Bot Personality menu ?
            else if (pUserMenu == &menuPODBotFillServerPersonality)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if ((atoi (arg1) >= 1) && (atoi (arg1) <= 3))
               {
                  g_cStoreFillServerPersonality = arg1[0];
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerTeam);
               }
               else if (FStrEq (arg1, "5"))
               {
                  g_cStoreFillServerPersonality = arg1[0];
                  UTIL_ShowMenu (pEntity, &menuPODBotFillServerTeam);
               }


               RETURN_META (MRES_SUPERCEDE);
            }

            // Fill Server Team Menu Select ?
            else if (pUserMenu == &menuPODBotFillServerTeam)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (atoi (arg1) >= 1 && (atoi (arg1) <= 2))
               {
                  g_cStoreFillServerTeam = arg1[0];
                  if (atoi (arg1) == TEAM_CS_TERRORIST)
                     UTIL_ShowMenu (pEntity, &menuPODBotFillServerTModel);
                  else // TEAM_CS_COUNTER
                     UTIL_ShowMenu (pEntity, &menuPODBotFillServerCTModel);
               }
               else if (FStrEq (arg1, "5"))
               {
                  PbCmdParser (g_pszPbCmds[PBCMD_FILLSERVER], g_cStoreFillServerSkill, &g_cStoreFillServerPersonality, arg1, NULL, NULL);
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Fill Server Model Menu Select ?
            else if ((pUserMenu == &menuPODBotFillServerTModel)
                     || (pUserMenu == &menuPODBotFillServerCTModel))
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if ((atoi (arg1) >= 1) && (atoi (arg1) <= 5))
               {
                  PbCmdParser (g_pszPbCmds[PBCMD_FILLSERVER], g_cStoreFillServerSkill, &g_cStoreFillServerPersonality, &g_cStoreFillServerTeam, arg1, NULL);
               }

               RETURN_META (MRES_SUPERCEDE);
            }

            // Kick Bot1 Select Menu ?
            else if (pUserMenu == &menuPODBotKickBot1)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (FStrEq (arg1, "1") || FStrEq (arg1, "2")
                  || FStrEq (arg1, "3") || FStrEq (arg1, "4")
                  || FStrEq (arg1, "5") || FStrEq (arg1, "6")
                  || FStrEq (arg1, "7") || FStrEq (arg1, "8"))
               {
                  memset (szTemp, 0, sizeof (szTemp));
                  i = atoi (arg1) - 1;
                  snprintf (szTemp, sizeof (szTemp), "%s", STRING (bots[i].pEdict->v.netname));
                  PbCmdParser (g_pszPbCmds[PBCMD_REMOVE], szTemp, NULL, NULL, NULL, NULL);
               }
               else if (FStrEq (arg1, "9"))
                  ShowPBKickBotMenu (pEntity, 2);
               else if (FStrEq (arg1, "10"))
                  UTIL_ShowMenu (pEntity, &menuPODBotMain);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Kick Bot2 Select Menu ?
            else if (pUserMenu == &menuPODBotKickBot2)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (FStrEq (arg1, "1") || FStrEq (arg1, "2")
                  || FStrEq (arg1, "3") || FStrEq (arg1, "4")
                  || FStrEq (arg1, "5") || FStrEq (arg1, "6")
                  || FStrEq (arg1, "7") || FStrEq (arg1, "8"))
               {
                  memset (szTemp, 0, sizeof (szTemp));
                  i = atoi (arg1) + 8 - 1;
                  snprintf (szTemp, sizeof (szTemp), "%s", STRING (bots[i].pEdict->v.netname));
                  PbCmdParser (g_pszPbCmds[PBCMD_REMOVE], szTemp, NULL, NULL, NULL, NULL);
               }
               else if (FStrEq (arg1, "9"))
                  ShowPBKickBotMenu (pEntity, 3);
               else if (FStrEq (arg1, "10"))
                  ShowPBKickBotMenu (pEntity, 1);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Kick Bot3 Select Menu ?
            else if (pUserMenu == &menuPODBotKickBot3)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (FStrEq (arg1, "1") || FStrEq (arg1, "2")
                  || FStrEq (arg1, "3") || FStrEq (arg1, "4")
                  || FStrEq (arg1, "5") || FStrEq (arg1, "6")
                  || FStrEq (arg1, "7") || FStrEq (arg1, "8"))
               {
                  memset (szTemp, 0, sizeof (szTemp));
                  i = atoi (arg1) + 16 - 1;
                  snprintf (szTemp, sizeof (szTemp), "%s", STRING (bots[i].pEdict->v.netname));
                  PbCmdParser (g_pszPbCmds[PBCMD_REMOVE], szTemp, NULL, NULL, NULL, NULL);
               }
               else if (FStrEq (arg1, "9"))
                  ShowPBKickBotMenu (pEntity, 4);
               else if (FStrEq (arg1, "10"))
                  ShowPBKickBotMenu (pEntity, 2);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Kick Bot4 Select Menu ?
            else if (pUserMenu == &menuPODBotKickBot4)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if (FStrEq (arg1, "1") || FStrEq (arg1, "2")
                  || FStrEq (arg1, "3") || FStrEq (arg1, "4")
                  || FStrEq (arg1, "5") || FStrEq (arg1, "6")
                  || FStrEq (arg1, "7") || FStrEq (arg1, "8"))
               {
                  memset (szTemp, 0, sizeof (szTemp));
                  i = atoi (arg1) + 24 - 1;
                  snprintf (szTemp, sizeof (szTemp), "%s", STRING (bots[i].pEdict->v.netname));
                  PbCmdParser (g_pszPbCmds[PBCMD_REMOVE], szTemp, NULL, NULL, NULL, NULL);
               }
               else if (FStrEq (arg1, "10"))
                  ShowPBKickBotMenu (pEntity, 3);

               RETURN_META (MRES_SUPERCEDE);
            }

            // Weapon Mode Selection Menu ?
            else if (pUserMenu == &menuPODBotWeaponMode)
            {
               UTIL_ShowMenu (pEntity, NULL); // reset menu display

               if ((atoi (arg1) >= 1) && (atoi (arg1) <= 7))
               {
                  PbCmdParser (g_pszPbCmds[PBCMD_WEAPONMODE], arg1, NULL, NULL, NULL, NULL);
               }

               RETURN_META (MRES_SUPERCEDE);
            }
         }

         RETURN_META (MRES_SUPERCEDE);
      }
   }

/*   // Check Kick Command  Could be useful If it was actually catchable..
   if (FStrEq (pcmd, "kick"))
   {
      if (arg1[0] == '#')
      {
         for (i = 0; i < gpGlobals->maxClients; ++i)
            if (bots[i].is_used && (GETPLAYERUSERID (bots[i].pEdict) == atoi (&arg1[1])))
               break;
         if (i < gpGlobals->maxClients)
            --g_iPeoBotsKept;
      }
      else
      {
         for (i = 0; i < gpGlobals->maxClients; ++i)
            if (bots[i].is_used && FStrEq (arg1, STRING (bots[i].pEdict->v.netname)))
               break;
         if (i < gpGlobals->maxClients)
            --g_iPeoBotsKept;
      }
      UTIL_ServerPrint ("PODBOT: kick command was called\n");
   }*/

   // Check Radio Commands
   iClientIndex = ENTINDEX (pEntity) - 1;

   if (clients[iClientIndex].IsAlive)
   {
      if ((iRadioSelect[iClientIndex] != 0) && FStrEq (pcmd, "menuselect"))
      {
         iRadioCommand = atoi (arg1);

         if (iRadioCommand != 0)
         {
            iRadioCommand += 10 * (iRadioSelect[iClientIndex] - 1);

            if ((iRadioCommand != RADIO_AFFIRMATIVE)
                && (iRadioCommand != RADIO_NEGATIVE)
                && (iRadioCommand != RADIO_REPORTINGIN))
            {
               for (i = 0; i < gpGlobals->maxClients; i++)
               {
                  if (bots[i].is_used && (bots[i].bot_team == clients[iClientIndex].iTeam)
                      && (pEntity != bots[i].pEdict))
                  {
                     if (bots[i].iRadioOrder == 0)
                     {
                        bots[i].iRadioOrder = iRadioCommand;
                        bots[i].pRadioEntity = pEntity;
                     }
                  }
               }
            }

            g_rgfLastRadioTime[clients[iClientIndex].iTeam - 1] = gpGlobals->time;
         }

         iRadioSelect[iClientIndex] = 0;
      }

      else if (strncmp (pcmd, "radio", 5) == 0)
         iRadioSelect[iClientIndex] = atoi (pcmd + 5);
   }
   // End Radio Commands

   RETURN_META (MRES_IGNORED);
}


void ServerActivate (edict_t *pEdictList, int edictCount, int clientMax)
{
   char line_buffer[256];
   int iBufferSize = sizeof (line_buffer);
   int i, c;
   int iChatType;
   replynode_t *pTempReply = NULL;
   replynode_t **pReply = NULL;
   char arg0[80];
   char arg1[80];
   int *ptrWeaponPrefs;
   int iParseWeapons;
   int iWeaponPrefsType;
   char *pszStart;
   char *pszEnd;
   STRINGNODE **pNode;
   STRINGNODE *pTempNode;
   char szDirectory[256];

   BotFreeAllMemory ();

   // Load & Initialise Botnames from 'Botnames.txt'
   snprintf (szDirectory, sizeof (szDirectory), "%s/addons/podbot/botnames.txt", g_szGameDirectory);
   fp = fopen (szDirectory, "r");
   if (fp == NULL)
      UTIL_ServerPrint ("POD-Bot couldn't find botnames.txt!\n");
   else
   {
//      memset (szBotNames, 0, sizeof (szBotNames));
      iNumBotNames = 0;

      while ((fgets (line_buffer, iBufferSize, fp) != NULL))
      {
         if ((line_buffer[0] == '#') || (line_buffer[0] == 0) || (line_buffer[0] == '\r') || (line_buffer[0] == '\n'))
            continue; // ignore comments or blank lines

         ++iNumBotNames;
      }

      g_pszBotNames = new botname_t[iNumBotNames];
      for (i = 0; i < iNumBotNames; ++i)
         memset (g_pszBotNames + i, 0, sizeof (g_pszBotNames));
      fseek (fp, 0, SEEK_SET);
      c = 0;

      while ((fgets (line_buffer, iBufferSize, fp) != NULL))
      {
         if ((line_buffer[0] == '#') || (line_buffer[0] == 0) || (line_buffer[0] == '\r') || (line_buffer[0] == '\n'))
            continue; // ignore comments or blank lines

         i = strlen (line_buffer);

         while ((line_buffer[i - 1] == '\n' || line_buffer[i - 1] == ' '
            || line_buffer[i - 1] == '\r' || line_buffer[i - 1] == '\t') && (i > 0))
            line_buffer[--i] = 0;

         strncpy ((g_pszBotNames + c)->name, line_buffer, sizeof (g_pszBotNames->name));
         (g_pszBotNames + c++)->name[sizeof (g_pszBotNames->name) - 1] = 0;
      }

      fclose (fp);
   }
   // End Botnames

   // Load & Initialise Botchats from 'Botchat.txt'
   snprintf (szDirectory, sizeof (szDirectory), "%s/addons/podbot/botchat.txt", g_szGameDirectory);
   fp = fopen (szDirectory, "r");
   if (fp == NULL)
      UTIL_ServerPrint ("POD-Bot couldn't find botchat.txt!\n");
   else
   {
      memset (szKillChat, 0, sizeof (szKillChat));
      memset (szBombChat, 0, sizeof (szKillChat));
      memset (szDeadChat, 0, sizeof (szKillChat));
      memset (szNoKwChat, 0, sizeof (szKillChat));
      iNumKillChats = 0;
      iNumBombChats = 0;
      iNumDeadChats = 0;
      iNumNoKwChats = 0;

      iChatType = -1;

      while (fgets (line_buffer, iBufferSize, fp) != NULL)
      {
         if ((line_buffer[0] == '#') || (line_buffer[0] == 0) || (line_buffer[0] == '\r') || (line_buffer[0] == '\n'))
            continue; // ignore comments or blank lines

         strncpy (arg0, GetField (line_buffer, 0), sizeof (arg0));
         arg0[sizeof (arg0) - 1] = 0;

         // Killed Chat Section ?
         if (FStrEq (arg0, "[KILLED]"))
         {
            iChatType = 0;
            continue;
         }

         // Bomb Chat Section ?
         else if (FStrEq (arg0, "[BOMBPLANT]"))
         {
            iChatType = 1;
            continue;
         }

         // Dead Chat Section ?
         else if (FStrEq (arg0, "[DEADCHAT]"))
         {
            iChatType = 2;
            continue;
         }

         // Keyword Chat Section ?
         else if (FStrEq (arg0, "[REPLIES]"))
         {
            iChatType = 3;
            pReply = &pChatReplies;
            continue;
         }

         // Unknown Keyword Section ?
         else if (FStrEq (arg0, "[UNKNOWN]"))
         {
            iChatType = 4;
            continue;
         }

         if (iChatType == 0)
         {
            strncat (line_buffer, "\n", iBufferSize);
            line_buffer[79] = 0;

            strncpy (szKillChat[iNumKillChats], line_buffer, sizeof (szKillChat[iNumKillChats]));
            iNumKillChats++;
         }

         else if (iChatType == 1)
         {
            strncat (line_buffer, "\n", iBufferSize);
            line_buffer[79] = 0;

            strncpy (szBombChat[iNumBombChats], line_buffer, sizeof (szBombChat[iNumBombChats]));
            iNumBombChats++;
         }

         else if (iChatType == 2)
         {
            strncat (line_buffer, "\n", iBufferSize);
            line_buffer[79] = 0;

            strncpy (szDeadChat[iNumDeadChats], line_buffer, sizeof (szDeadChat[iNumDeadChats]));
            iNumDeadChats++;
         }

         else if (iChatType == 3)
         {
            if (strstr (line_buffer, "@KEY") != NULL)
            {
               pTempReply = new (replynode_t);
               *pReply = pTempReply;
               pTempReply->pNextReplyNode = NULL;
               pTempReply->pReplies = NULL;
               pTempReply->cNumReplies = 0;
               pTempReply->cLastReply = 0;
               pNode = &pTempReply->pReplies;
               pTempNode = NULL;
               memset (pTempReply->szKeywords, 0, sizeof (pTempReply->szKeywords));

               c = 0;

               for (i = 0; i < iBufferSize; i++)
               {
                  if (line_buffer[i] == '\"')
                  {
                     i++;
                     while (line_buffer[i] != '\"')
                        pTempReply->szKeywords[c++] = line_buffer[i++];
                     pTempReply->szKeywords[c++] = '@';
                  }
                  else if (line_buffer[i] == 0)
                     break;
               }
               pReply = &pTempReply->pNextReplyNode;
            }
            else if (pTempReply)
            {
               strncat (line_buffer, "\n", iBufferSize);
               line_buffer[255] = 0;

               pTempNode = new (STRINGNODE);
               if (pTempNode == NULL)
                  UTIL_ServerPrint ("POD-Bot out of Memory!\n");
               else
               {
                  *pNode = pTempNode;
                  pTempNode->Next = NULL;
                  strncpy (pTempNode->szString, line_buffer, sizeof (pTempNode->szString));

                  pTempReply->cNumReplies++;
                  pNode = &pTempNode->Next;
               }
            }
         }

         else if (iChatType == 4)
         {
            strncat (line_buffer, "\n", iBufferSize);
            line_buffer[79] = 0;

            strncpy (szNoKwChat[iNumNoKwChats], line_buffer, sizeof (szNoKwChat[iNumNoKwChats]));
            iNumNoKwChats++;
         }
      }

      fclose (fp);
   }
   // End Botchats

   // Load & Initialise Botskill.cfg
   snprintf (szDirectory, sizeof (szDirectory), "%s/addons/podbot/botskill.cfg", g_szGameDirectory);
   fp = fopen (szDirectory, "r");
   if (fp == NULL)
      UTIL_ServerPrint ("No Botskill.cfg ! Using defaults...\n");
   else
   {
      i = 0;

      while (fgets (line_buffer, iBufferSize, fp) != NULL)
      {
         if ((line_buffer[0] == '#') || (line_buffer[0] == 0) || (line_buffer[0] == '\r') || (line_buffer[0] == '\n'))
            continue; // ignore comments or blank lines

         strncpy (arg0, GetField (line_buffer, 0), sizeof (arg0));
         strncpy (arg1, GetField (line_buffer, 1), sizeof (arg1));

         if (FStrEq (arg0, "MIN_DELAY"))
            BotSkillDelays[i].fMinSurpriseDelay = (float) atof (arg1);
         else if (FStrEq (arg0, "MAX_DELAY"))
            BotSkillDelays[i].fMaxSurpriseDelay = (float) atof (arg1);
         else if (FStrEq (arg0, "MIN_TURNSPEED"))
            BotTurnSpeeds[i].fMinTurnSpeed = (float) atof (arg1);
         else if (FStrEq (arg0, "MAX_TURNSPEED"))
            BotTurnSpeeds[i].fMaxTurnSpeed = (float) atof (arg1);
         else if (FStrEq (arg0, "AIM_OFFS_X"))
            BotAimTab[i].fAim_X = (float) atof (arg1);
         else if (FStrEq (arg0, "AIM_OFFS_Y"))
            BotAimTab[i].fAim_Y = (float) atof (arg1);
         else if (FStrEq (arg0, "AIM_OFFS_Z"))
            BotAimTab[i].fAim_Z = (float) atof (arg1);
         else if (FStrEq (arg0, "HEADSHOT_ALLOW"))
            BotAimTab[i].iHeadShot_Frequency = atoi (arg1);
         else if (FStrEq (arg0, "HEAR_SHOOTTHRU"))
            BotAimTab[i].iHeardShootThruProb = atoi (arg1);
         else if (FStrEq (arg0, "SEEN_SHOOTTHRU"))
         {
            BotAimTab[i].iSeenShootThruProb = atoi (arg1);

            if (i < 5)
               i++; // Prevent Overflow if Errors in cfg
         }
      }

      fclose (fp);
   }
   // End Botskill.cgf

   // Load & Initialise BotLogos from BotLogos.cfg
   snprintf (szDirectory, sizeof (szDirectory), "%s/addons/podbot/botlogos.cfg", g_szGameDirectory);
   fp = fopen (szDirectory, "r");
   if (fp == NULL)
      UTIL_ServerPrint ("No BotLogos.cfg ! Using Defaults...\n");
   else
   {
      g_iNumLogos = 0;

      while (fgets (line_buffer, iBufferSize, fp) != NULL)
      {
         if ((line_buffer[0] == '#') || (line_buffer[0] == 0) || (line_buffer[0] == '\r') || (line_buffer[0] == '\n'))
            continue; // ignore comments or blank lines

         strncpy (szSprayNames[g_iNumLogos], GetField (line_buffer, 0), sizeof (szSprayNames[g_iNumLogos]));
         g_iNumLogos++;
      }

      fclose (fp);
   }
   // End BotLogos

   // Load & initialise Weapon Stuff from 'BotWeapons.cfg'
   snprintf (szDirectory, sizeof (szDirectory), "%s/addons/podbot/botweapons.cfg", g_szGameDirectory);
   fp = fopen (szDirectory, "r");
   if (fp == NULL)
      UTIL_ServerPrint ("No BotWeapons.cfg ! Using Defaults...\n");
   else
   {
      iParseWeapons = 0;
      ptrWeaponPrefs = NULL;

      while (fgets (line_buffer, iBufferSize, fp) != NULL)
      {
         if ((line_buffer[0] == '#') || (line_buffer[0] == 0) || (line_buffer[0] == '\r') || (line_buffer[0] == '\n'))
            continue; // ignore comments or blank lines

         strncpy (arg0, GetField (line_buffer, 0), sizeof (arg0));

         if (iParseWeapons < 2)
         {
            if (FStrEq (arg0, "[STANDARD]"))
               iWeaponPrefsType = MAP_DE;
            else if (FStrEq (arg0, "[AS]"))
               iWeaponPrefsType = MAP_AS;
            else
            {
               pszStart = &line_buffer[0];
               pszEnd = NULL;

               if (iWeaponPrefsType == MAP_DE)
               {
                  for (i = 0; i < NUM_WEAPONS; i++)
                  {
                     pszEnd = strchr (pszStart, ',');
                     cs_weapon_select[i].iTeamStandard = atoi (pszStart);
                     pszStart = pszEnd + 1;
                  }
               }
               else
               {
                  for (i = 0; i < NUM_WEAPONS; i++)
                  {
                     pszEnd = strchr (pszStart, ',');
                     cs_weapon_select[i].iTeamAS = atoi (pszStart);
                     pszStart = pszEnd + 1;
                  }
               }

               iParseWeapons++;
            }
         }
         else
         {
            if (FStrEq (arg0, "[NORMAL]"))
               ptrWeaponPrefs = &NormalWeaponPrefs[0];
            else if (FStrEq (arg0, "[AGRESSIVE]"))
               ptrWeaponPrefs = &AgressiveWeaponPrefs[0];
            else if (FStrEq (arg0, "[DEFENSIVE]"))
               ptrWeaponPrefs = &DefensiveWeaponPrefs[0];
            else
            {
               pszStart = &line_buffer[0];

               for (i = 0; i < NUM_WEAPONS; i++)
               {
                  pszEnd = strchr (pszStart, ',');
                  *ptrWeaponPrefs++ = atoi (pszStart);
                  pszStart = pszEnd + 1;
               }
            }
         }
      }

      fclose (fp);
   }

   // Load the Waypoints for this Map
   WaypointLoad ();
   InitVisTab ();
   InitExperienceTab ();

   // Initialise the Client Struct for welcoming and keeping track who's connected
   memset (clients, 0, sizeof (clients));

   // Initialize the bots array of structures
   memset (bots, 0, sizeof (bots));

   // Initialize TheFatal's method for calculating the msec value
   msecnum = 0;
   msecdel = 0;
   msecval = 0;

   // a new map has started...
   botcreation_time = gpGlobals->time + g_flMapStartBotJoinDelay;

   // set the respawn time
   g_fTimeRoundStart = gpGlobals->time + CVAR_GET_FLOAT ("mp_freezetime");

   RETURN_META (MRES_IGNORED);
}


void ServerDeactivate (void)
{
   int index;
   int tab_index;

   tab_index = 0;
   for (index = 0; index < gpGlobals->maxClients; index++)
      if (bots[index].is_used)
      {
         BotCreateTab[tab_index].bNeedsCreation = TRUE;
         strncpy (BotCreateTab[tab_index].bot_name, bots[index].name, sizeof (BotCreateTab[tab_index].bot_name));
         BotCreateTab[tab_index].bot_skill = bots[index].bot_skill;
         BotCreateTab[tab_index].bot_personality = bots[index].bot_personality;
         BotCreateTab[tab_index].bot_team = bots[index].bot_team;
         BotCreateTab[tab_index].bot_class = bots[index].bot_class;
         tab_index++;
      }

   // Save collected Experience on Shutdown
   SaveExperienceTab ();
   SaveVisTab ();

   // Free everything that's freeable
   BotFreeAllMemory ();

   RETURN_META (MRES_IGNORED);
}


// Called each Server frame at the very beginning
void StartFrame (void)
{
   edict_t *pPlayer;
   static int i, player_index, bot_index;
   static FILE *bot_cfg_fp = NULL;
   static char cmd_line[1024];
   static char cmd[128];
   static char arg1[128];
   static char arg2[128];
   static char arg3[128];
   static char arg4[128];
   static char arg5[128];
   static float pause_time;
   static char szDirectory[256];

   if ((gpGlobals->time + 0.1) < previous_time)
   {
/*      for (i = 0; i < gpGlobals->maxClients; i++)
         if (BotCreateTab[i].bNeedsCreation)
            break;

      if (i < gpGlobals->maxClients)*/
         g_bIgnorePodCfgAddBot = TRUE;
/*      else
         g_bIgnorePodCfgAddBot = FALSE;*/
   }

   // Should the Map restart now ?
   if ((g_fTimeRestartServer > 0) && (g_fTimeRestartServer < gpGlobals->time))
   {
      g_fTimeRestartServer = 0; // don't keep restarting over and over again
      SERVER_COMMAND ("restart\n"); // restart the map
   }

   // Record some Stats of all Players on the Server
   num_players = 0;
   for (player_index = 0; player_index < gpGlobals->maxClients; player_index++)
   {
      pPlayer = INDEXENT (player_index + 1);

      if (!FNullEnt (pPlayer) && (pPlayer->v.flags & FL_CLIENT))
      {
         num_players++;
         clients[player_index].pEdict = pPlayer;
         clients[player_index].IsUsed = TRUE;
         clients[player_index].IsAlive = IsAlive (pPlayer);

         if (clients[player_index].IsAlive)
         {
            // Keep noclip on or else it might turn off after new round
            if ((pHostEdict == pPlayer) && g_bEditNoclip)
               pHostEdict->v.movetype = MOVETYPE_NOCLIP;

            clients[player_index].iTeam = UTIL_GetTeam (pPlayer);
            clients[player_index].vOrigin = pPlayer->v.origin;
            SoundSimulateUpdate (player_index);
         }

         // Does Client need to be shocked by the ugly red welcome message ?
         if ((clients[player_index].welcome_time > 0) && (clients[player_index].welcome_time < gpGlobals->time))
         {
            // Real Clients only
            if (!(pPlayer->v.flags & FL_FAKECLIENT))
            {
               // Hacked together Version of HUD_DrawString
               MESSAGE_BEGIN (MSG_ONE_UNRELIABLE, SVC_TEMPENTITY, NULL, pPlayer);
               WRITE_BYTE (TE_TEXTMESSAGE);
               WRITE_BYTE (1);
               WRITE_SHORT (FixedSigned16 (-1, 1 << 13));
               WRITE_SHORT (FixedSigned16 (0, 1 << 13));
               WRITE_BYTE (2);
               WRITE_BYTE (255);
               WRITE_BYTE (0);
               WRITE_BYTE (0);
               WRITE_BYTE (0);
               WRITE_BYTE (255);
               WRITE_BYTE (255);
               WRITE_BYTE (255);
               WRITE_BYTE (200);
               WRITE_SHORT (FixedUnsigned16 (0.0078125, 1 << 8));
               WRITE_SHORT (FixedUnsigned16 (2, 1 << 8));
               WRITE_SHORT (FixedUnsigned16 (6, 1 << 8));
               WRITE_SHORT (FixedUnsigned16 (0.1, 1 << 8));
               WRITE_STRING (szWelcomeMessage);
               MESSAGE_END ();

               // If this is the Host, scare him even more with a spoken Message
               if (g_bUseSpeech && (pPlayer == pHostEdict))
                  SERVER_COMMAND ((char *) &szSpeechSentences[RANDOM_LONG (0, 15)]);
            }

            clients[player_index].welcome_time = 0.0;
         }

         // Does Client need to be shocked by the ugly yellow waypoint message ?
         if ((clients[player_index].wptmessage_time > 0) && (clients[player_index].wptmessage_time < gpGlobals->time)
             && !(pPlayer->v.flags & FL_SPECTATOR))
         {
            // Real Clients only
            if (!(pPlayer->v.flags & FL_FAKECLIENT))
            {
               if (GET_USER_MSG_ID (PLID, "TextMsg", NULL) == 0)
                  REG_USER_MSG ("TextMsg", -1);

               MESSAGE_BEGIN (MSG_ONE_UNRELIABLE, GET_USER_MSG_ID (PLID, "TextMsg", NULL), NULL, pPlayer);
               WRITE_BYTE (HUD_PRINTCENTER);
               WRITE_STRING (g_szWaypointMessage);
               MESSAGE_END ();
            }

            clients[player_index].wptmessage_time = 0.0;
         }
      }
      else
      {
         clients[player_index].pEdict = NULL;
         clients[player_index].IsUsed = FALSE;
         clients[player_index].IsAlive = FALSE;
      }
   }

   // Estimate next Frame's duration
   EstimateNextFrameDuration ();

   // Go through all active Bots, calling their Think fucntion
   num_bots = 0;
   for (bot_index = 0; bot_index < gpGlobals->maxClients; bot_index++)
   {
      if (bots[bot_index].is_used
          && !FNullEnt (bots[bot_index].pEdict))
      {
         BotThink (&bots[bot_index]);
         num_bots++;
      }
   }

   // Show Waypoints to host if turned on and no dedicated Server
   if (!g_bIsDedicatedServer && g_bWaypointOn && !FNullEnt (pHostEdict))
      WaypointThink ();

   if (g_bMapInitialised && g_bRecalcVis)
      WaypointCalcVisibility ();

   // Keep it within the limits
   for (i = 0; i < gpGlobals->maxClients; i++)
      if (BotCreateTab[i].bNeedsCreation)
         break;
   if ((i >= gpGlobals->maxClients) && (botcreation_time == 0.0))
   {
      if ((g_iPeoBotsKept > max_bots) && (max_bots > 0))
         g_iPeoBotsKept = max_bots;
      if (g_iPeoBotsKept < min_bots)
         g_iPeoBotsKept = min_bots;
      if (max_bots == 0)
         g_iPeoBotsKept = num_players;
   }

   // Kick a bot if there is more than the maximum allowed
   if ((num_players > g_iPeoBotsKept) && (max_bots > 0) && (num_bots > min_bots)
      && (g_fLastKickedBotTime + 0.5 < gpGlobals->time))
      for (i = 0; i < gpGlobals->maxClients; i++)
         if (bots[i].is_used && !FNullEnt (bots[i].pEdict))
         {
            snprintf (cmd_line, sizeof (cmd_line), "kick \"%s\"\n", STRING (bots[i].pEdict->v.netname));
            SERVER_COMMAND (cmd_line);
            break;
         }

   // make sure it is allowed to create bots
   if (botcreation_time > 0.0)
   {
      // Don't allow creating Bots when max_bots is reached
      if ((num_players >= max_bots) && (max_bots > 0))
      {
         UTIL_ServerPrint ("Max Bots reached, can't create Bot !\n");
         memset (BotCreateTab, 0, sizeof (BotCreateTab));
         botcreation_time = 0.0;
      }

      // Don't allow creating Bots when no waypoints are loaded
      else if (g_iNumWaypoints < 1)
      {
         UTIL_ServerPrint ("No Waypoints for this Map, can't create Bot !\n");
         memset (BotCreateTab, 0, sizeof (BotCreateTab));
         botcreation_time = 0.0;
      }

      // if Waypoints have changed don't allow it because Distance Tables are messed up
      else if (g_bWaypointsChanged)
      {
         UTIL_ServerPrint ("Waypoints changed/not initialised, can't create Bot !\n");
         memset (BotCreateTab, 0, sizeof (BotCreateTab));
         botcreation_time = 0.0;
      }
   }
   else if ((gpGlobals->maxClients > num_players) && ((num_bots < min_bots) || (num_players < g_iPeoBotsKept))
           && (g_iNumWaypoints > 0) && !(g_bWaypointsChanged) && (iNumBotNames > num_bots) && (max_bots > 0))
   {
      for (i = 0; i < gpGlobals->maxClients; i++)
         if (!BotCreateTab[i].bNeedsCreation)
            break;
      if (i < gpGlobals->maxClients)
      {
         memset (&BotCreateTab[i], 0, sizeof (createbot_t));
         BotCreateTab[i].bot_skill = 101;
         BotCreateTab[i].bot_personality = 5;
         BotCreateTab[i].bot_team = 5;
         BotCreateTab[i].bot_class = 5;
         BotCreateTab[i].bNeedsCreation = TRUE;
         if (botcreation_time == 0.0)
            botcreation_time = gpGlobals->time;
      }
   }

   // are we currently spawning bots and is it time to spawn one yet?
   if ((botcreation_time > 0.0) && (botcreation_time < gpGlobals->time))
   {
      // find bot needing to be spawned...
      for (bot_index = 0; bot_index < gpGlobals->maxClients; bot_index++)
         if (BotCreateTab[bot_index].bNeedsCreation)
            break;

      if (bot_index < gpGlobals->maxClients)
      {
         botcreation_time = gpGlobals->time + 0.5; // set next spawn time
         BotCreate (BotCreateTab[bot_index].bot_skill, BotCreateTab[bot_index].bot_personality,
            BotCreateTab[bot_index].bot_team, BotCreateTab[bot_index].bot_class, BotCreateTab[bot_index].bot_name);
         memset (&BotCreateTab[bot_index], 0, sizeof (createbot_t));
      }
      else
         botcreation_time = 0.0;
   }

   // Executing the .cfg file - most work already done by Botman
   // Command Names are the same as in ClientCommand
   if (g_GameRules)
   {
      if (!file_opened)  // have we open podbot.cfg file yet?
      {
         UTIL_ServerPrint ("Executing podbot.cfg\n");

         snprintf (szDirectory, sizeof (szDirectory), "%s/addons/podbot/podbot.cfg", g_szGameDirectory);
         bot_cfg_fp = fopen (szDirectory, "r");
         if (bot_cfg_fp == NULL)
            UTIL_ServerPrint ("podbot.cfg File not found\n");

         file_opened = TRUE;
         pause_time = gpGlobals->time;
      }

      // if the bot.cfg file is still open and time to execute command...
      while ((bot_cfg_fp != NULL) && !feof (bot_cfg_fp) && (pause_time < gpGlobals->time))
      {
         if (fgets (cmd_line, sizeof (cmd_line), bot_cfg_fp) != NULL)
         {
            if ((cmd_line[0] == '#') || (cmd_line[0] == '\r') || (cmd_line[0] == '\n') || (cmd_line[0] == 0))
               continue; // ignore comments or blank lines

            snprintf (cmd, sizeof (cmd), GetField (cmd_line, 0));
            snprintf (arg1, sizeof (arg1), GetField (cmd_line, 1));
            snprintf (arg2, sizeof (arg2), GetField (cmd_line, 2));
            snprintf (arg3, sizeof (arg3), GetField (cmd_line, 3));
            snprintf (arg4, sizeof (arg4), GetField (cmd_line, 4));
            snprintf (arg5, sizeof (arg5), GetField (cmd_line, 5));

            if (FStrEq (cmd, CONSOLE_CMD_PAUSE))
            {
               pause_time = gpGlobals->time + atof (arg1);
               break;
            }
/*            else if (FStrEq (cmd, g_pszPbCmds[PBCMD_ADDBOT]) && g_bIgnorePodCfgAddBot)
            {
               break;
            }*/
            else if (FStrEq (cmd, g_pszPbCmds[PBCMD_ADDBOT])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_BOTCHAT])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_BOTSFOLLOWUSER])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_BOTSPRAY])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_DANGERFACTOR])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_DEBUGGOAL])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_DETAILNAMES])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_EXPERIENCE])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_FILLSERVER])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_HELP])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_INHUMANTURNS])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_JASONMODE])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_KILLBOTS])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_MAPSTARTBOTJOINDELAY])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_MAXBOTS])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_MAXBOTSKILL])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_MAXWEAPONPICKUP])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_MINBOTS])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_MINBOTSKILL])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_REMOVE])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_REMOVEBOTS])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_SHOOTTHRUWALLS])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_TIMERGRENADE])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_TIMERPICKUP])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_TIMERSOUND])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_USESPEECH])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_VERSION])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_WEAPONMODE])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_WELCOMEMESSAGE])
                    || FStrEq (cmd, g_pszPbCmds[PBCMD_WPTFOLDER])
            )
            {
               PbCmdParser (cmd, arg1, arg2, arg3, arg4, arg5);
               break;
            }
            else
            {
               UTIL_ServerPrint ("Executing server command: %s\n", cmd_line);
               strncat (cmd_line, "\n", sizeof (cmd_line));
               SERVER_COMMAND (cmd_line);
               break;
            }
         }
      }

      // if bot.cfg file is open and reached end of file, then close and free it
      if ((bot_cfg_fp != NULL) && feof (bot_cfg_fp))
      {
         fclose (bot_cfg_fp);
         bot_cfg_fp = NULL;
         g_GameRules = FALSE;

         // print the waypoint message to console
         UTIL_ServerPrint (g_szWaypointMessage);
      }
   }

   previous_time = gpGlobals->time;

   RETURN_META (MRES_IGNORED);
}


void FakeClientCommand (edict_t *pFakeClient, const char *fmt, ...)
{
   // the purpose of this function is to provide fakeclients (bots) with the same client
   // command-scripting advantages (putting multiple commands in one line between semicolons)
   // as real players. It is an improved version of botman's FakeClientCommand, in which you
   // supply directly the whole string as if you were typing it in the bot's "console". It
   // is supposed to work exactly like the pfnClientCommand (server-sided client command).

   va_list argptr;
   static char command[256];
   int length, fieldstart, fieldstop, i, index, stringindex = 0;
   int iClientIndex;
   int iRadioCommand;

   if (FNullEnt (pFakeClient))
      return; // reliability check

   // concatenate all the arguments in one string
   va_start (argptr, fmt);
   vsnprintf (command, sizeof (command), fmt, argptr);
   va_end (argptr);

   if ((command == NULL) || (*command == 0))
      return; // if nothing in the command buffer, return

   isFakeClientCommand = TRUE; // set the "fakeclient command" flag
   length = strlen (command); // get the total length of the command string

   // process all individual commands (separated by a semicolon) one each a time
   while (stringindex < length)
   {
      fieldstart = stringindex; // save field start position (first character)
      while ((stringindex < length) && (command[stringindex] != ';'))
         stringindex++; // reach end of field
      if (command[stringindex - 1] == '\n')
         fieldstop = stringindex - 2; // discard any trailing '\n' if needed
      else
         fieldstop = stringindex - 1; // save field stop position (last character before semicolon or end)
      for (i = fieldstart; i <= fieldstop; i++)
         g_argv[i - fieldstart] = command[i]; // store the field value in the g_argv global string
      g_argv[i - fieldstart] = 0; // terminate the string
      stringindex++; // move the overall string index one step further to bypass the semicolon

      index = 0;
      fake_arg_count = 0; // let's now parse that command and count the different arguments

      // count the number of arguments
      while (index < i - fieldstart)
      {
         while ((index < i - fieldstart) && (g_argv[index] == ' '))
            index++; // ignore spaces

         // is this field a group of words between quotes or a single word ?
         if (g_argv[index] == '"')
         {
            index++; // move one step further to bypass the quote
            while ((index < i - fieldstart) && (g_argv[index] != '"'))
               index++; // reach end of field
            index++; // move one step further to bypass the quote
         }
         else
            while ((index < i - fieldstart) && (g_argv[index] != ' '))
               index++; // this is a single word, so reach the end of field

         fake_arg_count++; // we have processed one argument more
      }

      // Check Radio Commands (fix): do it here since metamod won't call our own ClientCommand()
      iClientIndex = ENTINDEX (pFakeClient) - 1;

      if (clients[iClientIndex].IsAlive && (iRadioSelect[iClientIndex] != 0) && (strncmp (g_argv, "menuselect", 10) == 0))
      {
         iRadioCommand = atoi (g_argv + 11);

         if (iRadioCommand != 0)
         {
            iRadioCommand += 10 * (iRadioSelect[iClientIndex] - 1);

            if ((iRadioCommand != RADIO_AFFIRMATIVE)
                && (iRadioCommand != RADIO_NEGATIVE)
                && (iRadioCommand != RADIO_REPORTINGIN))
            {
               for (i = 0; i < gpGlobals->maxClients; i++)
               {
                  if (bots[i].is_used && (bots[i].bot_team == clients[iClientIndex].iTeam)
                      && (pFakeClient != bots[i].pEdict))
                  {
                     if (bots[i].iRadioOrder == 0)
                     {
                        bots[i].iRadioOrder = iRadioCommand;
                        bots[i].pRadioEntity = pFakeClient;
                     }
                  }
               }
            }

            g_rgfLastRadioTime[clients[iClientIndex].iTeam - 1] = gpGlobals->time;
         }

         iRadioSelect[iClientIndex] = 0;
      }
      else if (strncmp (g_argv, "radio", 5) == 0)
         iRadioSelect[iClientIndex] = atoi (g_argv + 5);

      // End Radio Commands

      MDLL_ClientCommand (pFakeClient); // tell now the MOD DLL to execute this ClientCommand...
   }

   g_argv[0] = 0; // when it's done, reset the g_argv field
   isFakeClientCommand = FALSE; // reset the "fakeclient command" flag
   fake_arg_count = 0; // and the argument count
}


const char *GetField (const char *string, int field_number)
{
   // This function gets and returns a particuliar field in a string where several fields are
   // concatenated. Fields can be words, or groups of words between quotes ; separators may be
   // white space or tabs. A purpose of this function is to provide bots with the same Cmd_Argv
   // convenience the engine provides to real clients. This way the handling of real client
   // commands and bot client commands is exactly the same, just have a look in engine.cpp
   // for the hooking of pfnCmd_Argc, pfnCmd_Args and pfnCmd_Argv, which redirects the call
   // either to the actual engine functions (when the caller is a real client), either on
   // our function here, which does the same thing, when the caller is a bot.

   static char field[256];
   int length, i, index = 0, field_count = 0, fieldstart, fieldstop;

   field[0] = 0; // reset field
   length = strlen (string); // get length of string

   while ((length > 0) && ((string[length - 1] == '\n') || (string[length - 1] == '\r')))
      length--; // discard trailing newlines

   // while we have not reached end of line
   while ((index < length) && (field_count <= field_number))
   {
      while ((index < length) && ((string[index] == ' ') || (string[index] == '\t')))
         index++; // ignore spaces or tabs

      // is this field multi-word between quotes or single word ?
      if (string[index] == '"')
      {
         index++; // move one step further to bypass the quote
         fieldstart = index; // save field start position
         while ((index < length) && (string[index] != '"'))
            index++; // reach end of field
         fieldstop = index - 1; // save field stop position
         index++; // move one step further to bypass the quote
      }
      else
      {
         fieldstart = index; // save field start position
         while ((index < length) && ((string[index] != ' ') && (string[index] != '\t')))
            index++; // reach end of field
         fieldstop = index - 1; // save field stop position
      }

      // is this field we just processed the wanted one ?
      if (field_count == field_number)
      {
         for (i = fieldstart; i <= fieldstop; i++)
            field[i - fieldstart] = string[i]; // store the field value in a string
         field[i - fieldstart] = 0; // terminate the string
         break; // and stop parsing
      }

      field_count++; // we have parsed one field more
   }

   return (&field[0]); // returns the wanted field
}


void PODBot_ServerCommand (void)
{
   char pcmd[128];
   char arg1[128];
   char arg2[128];
   char arg3[128];
   char arg4[128];
   char arg5[128];

   snprintf (pcmd, sizeof (pcmd), CMD_ARGV (1));
   snprintf (arg1, sizeof (arg1), CMD_ARGV (2));
   snprintf (arg2, sizeof (arg2), CMD_ARGV (3));
   snprintf (arg3, sizeof (arg3), CMD_ARGV (4));
   snprintf (arg4, sizeof (arg4), CMD_ARGV (5));
   snprintf (arg5, sizeof (arg5), CMD_ARGV (6));

   PbCmdParser (pcmd, arg1, arg2, arg3, arg4, arg5);
}


void PbCmdParser (const char *pcmd, const char *arg1, const char *arg2, const char *arg3, const char *arg4, const char *arg5)
{
   // This is cleaner than polling a CVAR, eh ? :) -- PM
   char kickcmd[80];
   int index;
   int iSkill, iPersonality, iTeam, iClass, iSelection;
   const char *const szPBszCVARReturn = "\"pb %s\" is \"%s\"\n";
   const char *const szPBiCVARReturn = "\"pb %s\" is \"%d\"\n";
   const char *const szPBfCVARReturn = "\"pb %s\" is \"%f\"\n";

   // "addbot" adds a bot to creation queue with specified args
   if (FStrEq (pcmd, g_pszPbCmds[PBCMD_ADDBOT]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         iSkill = atoi (arg1);
      else
         iSkill = 101;

      if ((arg2 != NULL) && (*arg2 != 0))
         iPersonality = atoi (arg2) - 1;
      else
         iPersonality = 5;

      if ((arg3 != NULL) && (*arg3 != 0))
         iTeam = atoi (arg3);
      else
         iTeam = 5;

      if ((arg4 != NULL) && (*arg4 != 0))
         iClass = atoi (arg4);
      else
         iClass = 5;

      for (index = 0; index < gpGlobals->maxClients; index++)
         if (!BotCreateTab[index].bNeedsCreation)
            break;
      if (index < gpGlobals->maxClients)
      {
         memset (&BotCreateTab[index], 0, sizeof (createbot_t));
         if ((arg5 != NULL) && (*arg4 != 0))
            strncpy (BotCreateTab[index].bot_name, arg5, sizeof (BotCreateTab[index].bot_name));
         BotCreateTab[index].bot_skill = iSkill;
         BotCreateTab[index].bot_personality = iPersonality;
         BotCreateTab[index].bot_team = iTeam;
         BotCreateTab[index].bot_class = iClass;
         BotCreateTab[index].bNeedsCreation = TRUE;
         if (botcreation_time == 0.0)
            botcreation_time = gpGlobals->time;
         ++g_iPeoBotsKept;
      }
   }

   // "botchat" bots will chat if on
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_BOTCHAT]))
   {
      if (FStrEq (arg1, "on") || FStrEq (arg1, "1"))
         g_bBotChat = TRUE;
      else if (FStrEq (arg1, "off") || FStrEq (arg1, "0"))
         g_bBotChat = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_BOTCHAT], g_bBotChat);
   }

   // "botsfollowuser"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_BOTSFOLLOWUSER]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         g_iMaxNumFollow = atoi (arg1);
      if (g_iMaxNumFollow < 0)
         g_iMaxNumFollow = 3;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_BOTSFOLLOWUSER], g_iMaxNumFollow);
   }

   // "botspray" bots spray decals if on
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_BOTSPRAY]))
   {
      if (FStrEq (arg1, "on") || FStrEq (arg1, "1"))
         g_bBotSpray = TRUE;
      else if (FStrEq (arg1, "off") || FStrEq (arg1, "0"))
         g_bBotSpray = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_BOTSPRAY], g_bBotSpray);
   }

   // "danger_factor"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_DANGERFACTOR]))
   {
      if (atoi (arg1) > 0)
         g_iDangerFactor = atoi (arg1);

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_DANGERFACTOR], g_iDangerFactor);
   }

   // "debuggoal" Forces Bots to use a specified Waypoint as a Goal
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_DEBUGGOAL]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         g_iDebugGoalIndex = atoi (arg1);

      // Bots ignore Enemies in Goal Debug Mode
      if ((g_iDebugGoalIndex >= 0) && (g_iDebugGoalIndex < g_iNumWaypoints))
         g_bIgnoreEnemies = TRUE;
      else
         g_bIgnoreEnemies = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_DEBUGGOAL], g_iDebugGoalIndex);
   }

   // "detailnames"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_DETAILNAMES]))
   {
      if (FStrEq (arg1, "on") || FStrEq (arg1, "1"))
         g_bDetailNames = TRUE;
      else if (FStrEq (arg1, "off") || FStrEq (arg1, "0"))
         g_bDetailNames = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_DETAILNAMES], g_bDetailNames);
   }

   // "experience"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_EXPERIENCE]))
   {
      if (FStrEq (arg1, "save"))
      {
         SaveExperienceTab ();
         SaveVisTab ();
      }
   }

   // "fillserver" Fills the server with bots
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_FILLSERVER]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         iSkill = atoi (arg1);
      else
         iSkill = 101;

      if ((arg2 != NULL) && (*arg2 != 0))
         iPersonality = atoi (arg2) - 1;
      else
         iPersonality = 5;

      if ((arg3 != NULL) && (*arg3 != 0))
      {
         iTeam = atoi (arg3);
         CVAR_SET_STRING ("mp_limitteams", "0");
         CVAR_SET_STRING ("mp_autoteambalance", "0");
      }
      else
         iTeam = 5;

      if ((arg4 != NULL) && (*arg4 != 0))
         iClass = atoi (arg4);
      else
         iClass = 5;

      for (index = 0; index < gpGlobals->maxClients; index++)
      {
         if (!BotCreateTab[index].bNeedsCreation)
         {
            memset (&BotCreateTab[index], 0, sizeof (createbot_t));
            BotCreateTab[index].bot_skill = iSkill;
            BotCreateTab[index].bot_personality = iPersonality;
            BotCreateTab[index].bot_team = iTeam;
            BotCreateTab[index].bot_class = iClass;
            BotCreateTab[index].bNeedsCreation = TRUE;
         }
      }
      g_iPeoBotsKept = (max_bots == 0) ? gpGlobals->maxClients : max_bots;

      if (botcreation_time == 0.0)
         botcreation_time = gpGlobals->time;
   }

   // "inhumanturns" turn on for instant turns
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_INHUMANTURNS]))
   {
      if (FStrEq (arg1, "on") || FStrEq (arg1, "1"))
         g_bInstantTurns = TRUE;
      else if (FStrEq (arg1, "off") || FStrEq (arg1, "0"))
         g_bInstantTurns = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_INHUMANTURNS], g_bInstantTurns);
   }

   // "jasonmode" Forces bots to use knives if on
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_JASONMODE]))
   {
      if (FStrEq (arg1, "on") || FStrEq (arg1, "1"))
         g_bJasonMode = TRUE;
      else if (FStrEq (arg1, "off") || FStrEq (arg1, "0"))
         g_bJasonMode = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_JASONMODE], g_bJasonMode);
   }

   // "killbots" kills all bots
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_KILLBOTS]))
   {
      for (index = 0; index < gpGlobals->maxClients; index++)
         if (bots[index].is_used && !bots[index].bDead)
         {
            // If a Bot gets killed it decreases his frags, so add 1 here to not give human
            // players an advantage using this command
            bots[index].pEdict->v.frags++;
            MDLL_ClientKill (bots[index].pEdict);
         }

      UTIL_HostPrint ("All Bots killed!\n");
   }

   // "mapstartbotdelay" delays bot joining at map start
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_MAPSTARTBOTJOINDELAY]))
   {
      if (arg1 != NULL)
         g_flMapStartBotJoinDelay = atof (arg1);

      if (g_flMapStartBotJoinDelay < 0.0)
         g_flMapStartBotJoinDelay = 0.0;

      UTIL_ServerPrint (szPBfCVARReturn, g_pszPbCmds[PBCMD_MAPSTARTBOTJOINDELAY], g_flMapStartBotJoinDelay);
   }

   // "max_bots"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_MAXBOTS]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         max_bots = atoi (arg1);
      if (max_bots > gpGlobals->maxClients)
         max_bots = gpGlobals->maxClients;
      if (max_bots < min_bots)
      {
         min_bots = max_bots;
         UTIL_ServerPrint ("max_bots less than min_bots. min_bots = %d\n", max_bots);
      }

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_MAXBOTS], max_bots);
   }

   // "maxbotskill"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_MAXBOTSKILL]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         g_iMaxBotSkill = atoi (arg1);
      if (g_iMaxBotSkill > 100)
         g_iMaxBotSkill = 100;
      if (g_iMaxBotSkill < g_iMinBotSkill)
         g_iMaxBotSkill = g_iMinBotSkill;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_MAXBOTSKILL], g_iMaxBotSkill);
   }

   // "maxweaponpickup"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_MAXWEAPONPICKUP]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         g_iMaxWeaponPickup = atoi (arg1);
      if (g_iMaxWeaponPickup < 0)
         g_iMaxWeaponPickup = 1;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_MAXWEAPONPICKUP], g_iMaxWeaponPickup);
   }

   // "min_bots"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_MINBOTS]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         min_bots = atoi (arg1);
      if (min_bots < 0)
         min_bots = 0;
      if (min_bots > max_bots)
      {
         max_bots = min_bots;
         UTIL_ServerPrint ("min_bots greater than max_bots. max_bots = %d\n", min_bots);
      }

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_MINBOTS], min_bots);
   }

   // "minbotskill"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_MINBOTSKILL]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         g_iMinBotSkill = atoi (arg1);
      if (g_iMinBotSkill < 1)
         g_iMinBotSkill = 1;
      if (g_iMinBotSkill > g_iMaxBotSkill)
         g_iMinBotSkill = g_iMaxBotSkill;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_MINBOTSKILL], g_iMinBotSkill);
   }

   // "remove" removes a bot
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_REMOVE]))
   {
      if (arg1[0] == '#')
      {
         for (index = 0; index < gpGlobals->maxClients; ++index)
            if (bots[index].is_used && !FNullEnt (bots[index].pEdict)
               && (GETPLAYERUSERID (bots[index].pEdict) == atoi (&arg1[1])))
            {
               snprintf (kickcmd, sizeof (kickcmd), "kick \"%s\"\n", STRING (bots[index].pEdict->v.netname));
               SERVER_COMMAND (kickcmd); // kick the bot using (kick "name")
               --g_iPeoBotsKept;
               g_fLastKickedBotTime = gpGlobals->time;
            }
      }
      else
      {
         for (index = 0; index < gpGlobals->maxClients; ++index)
            if (bots[index].is_used && !FNullEnt (bots[index].pEdict)
               && !strcmpi (arg1, STRING (bots[index].pEdict->v.netname)))
            {
               snprintf (kickcmd, sizeof (kickcmd), "kick \"%s\"\n", STRING (bots[index].pEdict->v.netname));
               SERVER_COMMAND (kickcmd); // kick the bot using (kick "name")
               --g_iPeoBotsKept;
               g_fLastKickedBotTime = gpGlobals->time;
            }
      }

/*      if ((arg1 != NULL) && (*arg1 != 0))
         iTeam = atoi (arg1);
      else
         iTeam = 0;

      if ((arg2 != NULL) && (*arg2 != 0))
         iSelection = atoi (arg2);
      else
         iSelection = 0;

      if ((iTeam == TEAM_CS_TERRORIST) || (iTeam == TEAM_CS_COUNTER))
      {
         for (index = 0; index < gpGlobals->maxClients; index++)
            if (bots[index].is_used && !FNullEnt (bots[index].pEdict) && (bots[index].bot_team == iTeam))
            {
               --g_iPeoBotsKept;
               snprintf (kickcmd, sizeof (kickcmd), "kick \"%s\"\n", STRING (bots[index].pEdict->v.netname));
               SERVER_COMMAND (kickcmd);
               if (index >= iSelection)
                  break;
            }
      }
      else
      {
         for (index = 0; index < gpGlobals->maxClients; index++)
            if (bots[index].is_used && !FNullEnt (bots[index].pEdict))
            {
               --g_iPeoBotsKept;
               snprintf (kickcmd, sizeof (kickcmd), "kick \"%s\"\n", STRING (bots[index].pEdict->v.netname));
               SERVER_COMMAND (kickcmd);
               if (index >= iSelection)
                  break;
            }
      }*/
   }

   // "removebots" removes all bots
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_REMOVEBOTS]))
   {
      g_iPeoBotsKept = ((min_bots == 0) && !g_bIsDedicatedServer)? 1 : min_bots;

      for (index = 0; index < gpGlobals->maxClients; index++)
      {
         // Reset our Creation Tab if there are still Bots waiting to be spawned
         memset (&BotCreateTab[index], 0, sizeof (createbot_t));
         botcreation_time = 0.0;

         // is this slot used?
         if (bots[index].is_used && !FNullEnt (bots[index].pEdict))
         {
            snprintf (kickcmd, sizeof (kickcmd), "kick \"%s\"\n", STRING (bots[index].pEdict->v.netname));
            SERVER_COMMAND (kickcmd); // kick the bot using (kick "name")
         }
      }

      UTIL_HostPrint ("All Bots removed!\n");
   }

   // "shootthruwalls"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_SHOOTTHRUWALLS]))
   {
      if (FStrEq (arg1, "on") || FStrEq (arg1, "1"))
         g_bShootThruWalls = TRUE;
      else if (FStrEq (arg1, "off") || FStrEq (arg1, "0"))
         g_bShootThruWalls = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_SHOOTTHRUWALLS], g_bShootThruWalls);
   }

   // "timer_sound"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_TIMERSOUND]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         g_fTimeSoundUpdate = atof (arg1);
      if (g_fTimeSoundUpdate < 0.1)
         g_fTimeSoundUpdate = 0.1;

      UTIL_ServerPrint (szPBfCVARReturn, g_pszPbCmds[PBCMD_TIMERSOUND], g_fTimeSoundUpdate);
   }

   // "timer_pickup"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_TIMERPICKUP]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         g_fTimePickupUpdate = atof (arg1);
      if (g_fTimePickupUpdate < 0.1)
         g_fTimePickupUpdate = 0.1;

      UTIL_ServerPrint (szPBfCVARReturn, g_pszPbCmds[PBCMD_TIMERPICKUP], g_fTimePickupUpdate);
   }

   // "timer_grenade"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_TIMERGRENADE]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         g_fTimeGrenadeUpdate = atof (arg1);
      if (g_fTimeGrenadeUpdate < 0.1)
         g_fTimeGrenadeUpdate = 0.1;

      UTIL_ServerPrint (szPBfCVARReturn, g_pszPbCmds[PBCMD_TIMERGRENADE], g_fTimeGrenadeUpdate);
   }

   // "usespeech"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_USESPEECH]))
   {
      if (FStrEq (arg1, "on") || FStrEq (arg1, "1"))
         g_bUseSpeech = TRUE;
      else if (FStrEq (arg1, "off") || FStrEq (arg1, "0"))
         g_bUseSpeech = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_USESPEECH], g_bUseSpeech);
   }

   // "version"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_VERSION]))
   {
      UTIL_ServerPrint (szPBszCVARReturn, g_pszPbCmds[PBCMD_VERSION], Plugin_info.version);
   }

   // "weaponmode"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_WEAPONMODE]))
   {
      if ((atoi (arg1) >= 1) && (atoi (arg1) <= 7))
      {
         iSelection = atoi (arg1);

         int rgiWeaponTabStandard[7][NUM_WEAPONS] =
         {
            {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1}, // Knife only
            {-1,-1,-1, 2, 2, 0, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1}, // Pistols only
            {-1,-1,-1,-1,-1,-1,-1, 2, 2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1}, // Shotgun only
            {-1,-1,-1,-1,-1,-1,-1,-1,-1, 2, 1, 2, 0, 2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 2,-1}, // Machine Guns only
            {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 1, 0, 1, 1,-1,-1,-1,-1,-1,-1}, // Rifles only
            {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 2, 2, 0, 1,-1,-1}, // Snipers only
            {-1,-1,-1, 2, 2, 0, 1, 2, 2, 2, 1, 2, 0, 2, 0, 0, 1, 0, 1, 1, 2, 2, 0, 1, 2, 1}  // Standard
         };
         int rgiWeaponTabAS[7][NUM_WEAPONS] =
         {
            {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1}, // Knife only
            {-1,-1,-1, 2, 2, 0, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1}, // Pistols only
            {-1,-1,-1,-1,-1,-1,-1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1}, // Shotgun only
            {-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 0, 2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1,-1}, // Machine Guns only
            {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0,-1, 1, 0, 1, 1,-1,-1,-1,-1,-1,-1}, // Rifles only
            {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0,-1, 1,-1,-1}, // Snipers only
            {-1,-1,-1, 2, 2, 0, 1, 1, 1, 1, 1, 1, 0, 2, 0,-1, 1, 0, 1, 1, 0, 0,-1, 1, 1, 1}  // Standard
         };

         char szMode[7][12] =
         {
            {"Knife"},
            {"Pistol"},
            {"Shotgun"},
            {"Machine Gun"},
            {"Rifle"},
            {"Sniper"},
            {"Standard"}
         };

         iSelection--;
         for (index = 0; index < NUM_WEAPONS; index++)
         {
            cs_weapon_select[index].iTeamStandard = rgiWeaponTabStandard[iSelection][index];
            cs_weapon_select[index].iTeamAS = rgiWeaponTabAS[iSelection][index];
         }

         if (iSelection == 0)
            g_bJasonMode = TRUE;
         else
            g_bJasonMode = FALSE;

         UTIL_HostPrint ("%s Weapon Mode selected!\n", &szMode[iSelection][0]);
      }
      else
         UTIL_ServerPrint ("weaponmode must be between 1 and 7\n");
   }

   // "welcome_msgs"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_WELCOMEMESSAGE]))
   {
      if (FStrEq (arg1, "on") || FStrEq (arg1, "1"))
         g_bShowWelcomeMessages = TRUE;
      else if (FStrEq (arg1, "off") || FStrEq (arg1, "0"))
         g_bShowWelcomeMessages = FALSE;

      UTIL_ServerPrint (szPBiCVARReturn, g_pszPbCmds[PBCMD_WELCOMEMESSAGE], g_bShowWelcomeMessages);
   }

   // "wptfolder"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_WPTFOLDER]))
   {
      if ((arg1 != NULL) && (*arg1 != 0))
         strncpy (g_szWPTDirname, arg1, sizeof (g_szWPTDirname));

      UTIL_ServerPrint (szPBszCVARReturn, g_pszPbCmds[PBCMD_WPTFOLDER], g_szWPTDirname);
   }

   // "help"
   else if (FStrEq (pcmd, g_pszPbCmds[PBCMD_HELP]))
   {
      if (FStrEq (arg1, g_pszPbCmds[PBCMD_ADDBOT]))
         UTIL_ServerPrint ("");
      else
         UTIL_ServerPrint ("Podbot mm command list:\n"
                           "  All commands must be preceded with \"%s\"\n"
                           "    For example: \"%s %s\"\n"
                           "\n"
                           "  \"%s\"   \"%s\"   \"%s\"   \"%s\"\n"
                           "  \"%s\"   \"%s\"   \"%s\"   \"%s\"\n"
                           "  \"%s\"   \"%s\"   \"%s\"   \"%s\"\n"
                           "  \"%s\"   \"%s\"   \"%s\"   \"%s\"\n"
                           "  \"%s\"   \"%s\"   \"%s\"   \"%s\"\n"
                           "  \"%s\"   \"%s\"   \"%s\"   \"%s\"\n"
                           "  \"%s\"   \"%s\"   \"%s\"   \"%s\"\n"
                           "  \"%s\"   \"%s\"\n"
                           "Type \"%s %s <command>\" for more information.\n",
                           g_pszPbCmds[PBCMD],
                           g_pszPbCmds[PBCMD],
                           g_pszPbCmds[PBCMD_ADDBOT],
                           g_pszPbCmds[PBCMD_ADDBOT],
                           g_pszPbCmds[PBCMD_BOTCHAT],
                           g_pszPbCmds[PBCMD_BOTSFOLLOWUSER],
                           g_pszPbCmds[PBCMD_BOTSPRAY],
                           g_pszPbCmds[PBCMD_DANGERFACTOR],
                           g_pszPbCmds[PBCMD_DEBUGGOAL],
                           g_pszPbCmds[PBCMD_DETAILNAMES],
                           g_pszPbCmds[PBCMD_EXPERIENCE],
                           g_pszPbCmds[PBCMD_FILLSERVER],
                           g_pszPbCmds[PBCMD_HELP],
                           g_pszPbCmds[PBCMD_INHUMANTURNS],
                           g_pszPbCmds[PBCMD_JASONMODE],
                           g_pszPbCmds[PBCMD_KILLBOTS],
                           g_pszPbCmds[PBCMD_MAPSTARTBOTJOINDELAY],
                           g_pszPbCmds[PBCMD_MAXBOTS],
                           g_pszPbCmds[PBCMD_MAXBOTSKILL],
                           g_pszPbCmds[PBCMD_MAXWEAPONPICKUP],
                           g_pszPbCmds[PBCMD_MINBOTS],
                           g_pszPbCmds[PBCMD_MINBOTSKILL],
                           g_pszPbCmds[PBCMD_REMOVE],
                           g_pszPbCmds[PBCMD_REMOVEBOTS],
                           g_pszPbCmds[PBCMD_SHOOTTHRUWALLS],
                           g_pszPbCmds[PBCMD_TIMERGRENADE],
                           g_pszPbCmds[PBCMD_TIMERPICKUP],
                           g_pszPbCmds[PBCMD_TIMERSOUND],
                           g_pszPbCmds[PBCMD_USESPEECH],
                           g_pszPbCmds[PBCMD_VERSION],
                           g_pszPbCmds[PBCMD_WEAPONMODE],
                           g_pszPbCmds[PBCMD_WELCOMEMESSAGE],
                           g_pszPbCmds[PBCMD_WPTFOLDER],
                           g_pszPbCmds[PBCMD],
                           g_pszPbCmds[PBCMD_HELP]);
   }


   // If it were none of the above commands tell the user
   else
      UTIL_ServerPrint ("\"%s\" is not a Podbot command. Type \"%s %s\" for a list of commands\n", pcmd, g_pszPbCmds[PBCMD], g_pszPbCmds[PBCMD_HELP]);

   return;
}


void UserNewroundAll (void)
{
   // Called by UserCommand "newround"
   // Kills all Clients in Game including humans

   edict_t *pPlayer;

   for (int i = 1; i <= gpGlobals->maxClients; i++)
   {
      pPlayer = INDEXENT (i);

      // is this player slot valid
      if (!FNullEnt (pPlayer) && (pPlayer->v.flags & FL_CLIENT) && IsAlive (pPlayer))
      {
         pPlayer->v.frags++;
         MDLL_ClientKill (pPlayer);
      }
   }

   UTIL_HostPrint ("Round Restarted !\n");
}


void ShowPBKickBotMenu (edict_t *pEntity, int iMenuNum)
{
   if ((iMenuNum > 4) || (iMenuNum < 1))
      return;

   char szTemp1[1024], szTemp2[1024];
   int i = 0, iValidSlots = 0;
   memset (szTemp1, 0, sizeof (szTemp1));
   memset (szTemp2, 0, sizeof (szTemp2));
   iValidSlots = (iMenuNum == 4)? (1 << 9) : ((1 << 8) | (1 << 9));
   for (i = ((iMenuNum - 1) * 8); i < iMenuNum * 8; ++i)
   {
      if (bots[i].is_used && !FNullEnt (bots[i].pEdict))
      {
         iValidSlots |= 1 << (i - ((iMenuNum - 1) * 8));
         snprintf (szTemp2, sizeof (szTemp2), "%s %1.1d. %s\n", szTemp2, i - ((iMenuNum - 1) * 8) + 1, STRING (bots[i].pEdict->v.netname));
      }
      else
      {
         snprintf (szTemp2, sizeof (szTemp2), "%s\\d %1.1d. Not a PodbotMM\\w\n", szTemp2, i - ((iMenuNum - 1) * 8) + 1);
      }
   }
   snprintf (szTemp1, sizeof (szTemp1),
      " \\yKick Bot Menu#%d:\\w\n"
      "\n"
      "\n"
      "%s"
      "\n"
      "%s"
      " 0. Back",
      iMenuNum,
      szTemp2,
      (iMenuNum == 4)? "" : " 9. More...\n");
   if (iMenuNum == 1)
   {
      menuPODBotKickBot1.ValidSlots = iValidSlots & ((signed int)-1);
      menuPODBotKickBot1.szMenuText = szTemp1;
      UTIL_ShowMenu (pEntity, &menuPODBotKickBot1);
   }
   else if (iMenuNum == 2)
   {
      menuPODBotKickBot2.ValidSlots = iValidSlots & ((signed int)-1);
      menuPODBotKickBot2.szMenuText = szTemp1;
      UTIL_ShowMenu (pEntity, &menuPODBotKickBot2);
   }
   else if (iMenuNum == 3)
   {
      menuPODBotKickBot3.ValidSlots = iValidSlots & ((signed int)-1);
      menuPODBotKickBot3.szMenuText = szTemp1;
      UTIL_ShowMenu (pEntity, &menuPODBotKickBot3);
   }
   else if (iMenuNum == 4)
   {
      menuPODBotKickBot4.ValidSlots = iValidSlots & ((signed int)-1);
      menuPODBotKickBot4.szMenuText = szTemp1;
      UTIL_ShowMenu (pEntity, &menuPODBotKickBot4);
   }
   return;
}


void GetGameDir (void)
{
   // This function fixes the erratic behaviour caused by the use of the GET_GAME_DIR engine
   // macro, which returns either an absolute directory path, or a relative one, depending on
   // whether the game server is run standalone or not. This one always return a RELATIVE path.

   unsigned char length, fieldstart, fieldstop;

   GET_GAME_DIR (g_szGameDirectory); // call the engine macro and let it mallocate for the char pointer

   length = strlen (g_szGameDirectory); // get the length of the returned string
   length--; // ignore the trailing string terminator

   // format the returned string to get the last directory name
   fieldstop = length;
   while (((g_szGameDirectory[fieldstop] == '\\') || (g_szGameDirectory[fieldstop] == '/')) && (fieldstop > 0))
      fieldstop--; // shift back any trailing separator

   fieldstart = fieldstop;
   while ((g_szGameDirectory[fieldstart] != '\\') && (g_szGameDirectory[fieldstart] != '/') && (fieldstart > 0))
      fieldstart--; // shift back to the start of the last subdirectory name

   if ((g_szGameDirectory[fieldstart] == '\\') || (g_szGameDirectory[fieldstart] == '/'))
      fieldstart++; // if we reached a separator, step over it

   // now copy the formatted string back onto itself character per character
   for (length = fieldstart; length <= fieldstop; length++)
      g_szGameDirectory[length - fieldstart] = g_szGameDirectory[length];
   g_szGameDirectory[length - fieldstart] = 0; // terminate the string

   return;
}


C_DLLEXPORT int GetEntityAPI2 (DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion)
{
   gFunctionTable.pfnGameInit = GameDLLInit;
   gFunctionTable.pfnSpawn = Spawn;
   gFunctionTable.pfnClientConnect = ClientConnect;
   gFunctionTable.pfnClientDisconnect = ClientDisconnect;
   gFunctionTable.pfnClientPutInServer = ClientPutInServer;
   gFunctionTable.pfnClientCommand = ClientCommand;
   gFunctionTable.pfnServerActivate = ServerActivate;
   gFunctionTable.pfnServerDeactivate = ServerDeactivate;
   gFunctionTable.pfnStartFrame = StartFrame;

   memcpy (pFunctionTable, &gFunctionTable, sizeof (DLL_FUNCTIONS));
   return (TRUE);
}


C_DLLEXPORT int GetEntityAPI2_Post (DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion)
{
   gFunctionTable_Post.pfnSpawn = Spawn_Post;

   memcpy (pFunctionTable, &gFunctionTable_Post, sizeof (DLL_FUNCTIONS));
   return (TRUE);
}
